#!/usr/bin/env perl
# collect monitoring output and write to timestamped files
# run this script as the DB2 instance owner

my $version  = "3.14.3";
my $filename = $0;
my $Debug    = 0 ;  # By default detailed messages have been turned off
my $prompt   = 0 ;  # run in the background
my $instrumentation = "Y" ;
my ($administrator, $password) ;

my $startdir = getcwd() . "/";
my $rootMode = 0 ;
my $lastRootFile = "" ;

use POSIX("strftime");
use POSIX qw( WNOHANG) ;
use Cwd;
use List::Util qw(max);
use IO::File;
use IO::Seekable;
use File::Copy;
use File::Path;
use Carp;
use File::Basename ;
use Sys::Hostname ;
use Time::Local ;

# Check for File::Path
eval {
    require File::Path;
    File::Path->import(qw(remove_tree));
#    $deldir = 1 if $File::Path::VERSION >= 2.07;
};

if ( ( $^O eq "MSWin32" ) || ( $^O eq "Windows_NT" ) ) {
    require Win32API::Registry;
    Win32API::Registry->import(qw(KEY_READ));
    require Win32::TieRegistry;
}

#
#   Get username of executing user
#

my $username = "" ;

if ( ( $^O eq "MSWin32" ) || ( $^O eq "Windows_NT" ) ) 
    {
        $username = Win32::LoginName;   
    } else
        {
            $username = POSIX::cuserid() ;
        }

#
#   Before running ServerMon for the first time you will need to have created and TSM username and password. These will be stored in a file called servermon.txt, in an unreadable format.
#   Building the crendetials can be done by running command "perl servermon.pl buildcredentials"
#

#
#   ServerMon must be run using the DB2 user that was specified when TSM was installed. If not the utility till stop
#

#
#   Read the passed parameters and process. If we receive an invalid parameter, ServerMon will stop
#
if($#ARGV+1 > 0)
    {
        my $valid_flag   = 1 ;
        my $invalid_parm = "" ;
        
        foreach my $commandline(@ARGV)
            {               
                if( $commandline =~ /^buildcredentials$/i )
                    {
                        build_credentials() ;
                        next ;
                    }

                if( $commandline =~ /^show-details$/i )
                    {
                        show_details() ;
                        next ;
                    }

                if( $commandline =~ /^prompt$/i )
                    {
                        prompt() ;
                        next ;
                    }  
                    
                if( $commandline =~ /^noprompt$/i )  #  We just add this to allow it as a possible response
                    {
                        next ;
                    } 

                if( $commandline =~ /^root-mode$/i )
                    {
                        $rootMode = 1;
                        next ;
                    }
                    
                $valid_flag = 0 ;
                $invalid_parm = $commandline ;
            }
            
        if(!$valid_flag)
            {
                print "An unknown paramter \"$invalid_parm\" has been passed to ServerMon. Valid parameters are \"show-details\", \"buildcredentials\" and \"no-prompt\". Please restart ServerMon with one of these parameters or leave blank\n" ;
                exit 1 ;
            }
    }


START: 

my $IT21957         	= 0;    # exposed to SHOW THREADS hanging the server?
my $cyclesCompleted 	= 0;    # count completed diag collection cycles
my $stopping            = "interrupted";    # used in stopScript if SIG(INT) received
my $collectstmt 	 ;    # available with 6.2.2 and newer servers, enabled via $sqlstats = Y
my $statsinterval       = 30;    # interval length (sec) for iostat (UNIX only)
my $is_sta_status       = 2;
my $storageagent        = 0;    # are we talking to a Storage Agent?  
my $hasSDpools_status   = 2;
my $thisOperatingSystem = $^O;
my $tsm_available       = $instancefound = 0 ;
my ( $actlogname, $arc_year, $arc_month, $arc_day, $arc_hour, $arc_minute, $begindate, $begintime, $db2ZipProgram ) ;
my ( $summaryname, $trccmdname, $trcname, $begints, $now, $csvcycle, $csvdiag, $targetdir ) ;
my $archivecmd          = $nfsstatscmd = $vmstatcmd = $lparstatcmd = $lparstatcmdfinal = $pathdelimiter = "";
my $prefix 		= strftime( "%Y%m%d-%H%M", localtime() ) . "-" . sprintf( "%08d", "1" );
my $first_pass  	= 1;
my $hostname 		= hostname();
my $serverName          = "" ;
my $keep_log_open       = 1 ;


if ( $startdir =~ /system32/i ) 
    {
        ( $file, $startdir ) = fileparse($0);    # Added MW 0603
    }

#
#	Read the ServerMon ini file
#

read_ini_files("$startdir", "servermon.ini");

#	
# Build TSM commands into arrays from external files
#

load_command_variables();

#
#   If we have been passed the "admin-command" parameter branch to teh subroutine, run the commands and then exit
#

if ($rootMode == 1)
    {
        if ($db2User eq $username)
            {
                LogToBoth("\n\nUnable to run root commands as instance owner. To run root commands, switch to root user and rerun.\nServerMon will now exit") ;
                exit 55 ;
			}

        Debug_Message("Running root-commands\n") ;

        RootMessageHeader() ;

        $now = strftime( "%Y%m%d-%H%M", localtime() );
        $rootCommandsOutputFile 	= "$now-00000000-root-commands.txt" ;

        ProcessEndOfSection($rootCommandsOutputFile, @root_commands) ;
		if ( $? == 0)
			{
				findRootCommandFiles(1) ;
			}

        LogToBoth(".\n\n") ;

        exit 44;
    }

#
#   If 
#
        
if ( !(-e "servermon.pwd") and !$prompt)
    {
		print("We are here\n");
        build_credentials() ;
		LogToBoth("We are here 2\n");
    }   

#
#   Get predfined Protect access details
#

if ( !$prompt )
    {
        getCredentials(); 
    }

#
#   If we are in prompt mode, ask for credentials
#

if ( $prompt ) 
    {
    
        MessagesLogHandler(1) ;

        print "\nPress enter to accept default values shown in brackets:\n\n";
        $server = getResp( "Enter the servername as found in dsm.opt or dsm.sys", $server );

        if (   ( $^O eq "MSWin32" ) || ( $^O eq "Windows_NT" )  || ( $^O eq "cygwin" ) )
                {
                    $tcpport = getResp( "Enter the tcpport", $tcpport );
                }

        $administrator = getResp( "Enter the IBM Spectrum Protect administrator login ID", $administrator );
        $password 	   = readPassword( "Enter the IBM Spectrum Protect administrator password", $password );

        #$pmr = getResp( "Enter PMR number as PPPPP,BBB,CCC", $pmr );

        if ( !$showthreads ) { $showthreads = "Y"; }
            $showthreads = uc( getResp( "Collect SHOW THREADS output? (Y/N)", $showthreads ) );

        if ( !$showdedupdelete ) { $showdedupdelete = "Y"; }
            $showdedupdelete = uc( getResp( "Collect SHOW DEDUPDELETE output? (Y/N)", $showdedupdelete ) );

        if ( !$showmemory ) { $showmemory = "N"; }
            $showmemory = uc( getResp( "Collect SHOW for memory statistics? (Y/N)", $showmemory ) );

        if ( !$showreplication ) { $showreplication = "N"; }
            $showreplication = uc( getResp( "Collect SHOW REPLICATION output? (Y/N)", $showreplication ) );

        if ( !$ProcStack ) { $ProcStack = "N"; }
            $ProcStack = uc( getResp( "Run Procstack command? (Y/N)", $ProcStack ) );
            
        #
        #   Set AlwaysOn to N and request duration
        #
        
        $Duration = 24 ;    #   We're going to override the servermon.ini setting when in prompt mode
        $AlwaysOn = 'N' ;
        
        $AlwaysOn = uc( getResp( "Will Servermon run continuously (Y/N)", $AlwaysOn  )) ;
		
        if ($AlwaysOn eq "Y") {
                $CollCycleEndTime = uc( getResp("When will ServerMon start a new cycle ( format HH:MM )?", $CollCycleEndTime )) ;
            } else 
                    {
                        $Duration = uc( getResp("How many hours will ServerMon run for?", $Duration )) ;
                    }

        my $specified = 0;

        while ( !($specified) ) 
            {
                if ( $sqlstats eq "" ) 
                    {
                        $sqlstats = "N";
                    }

                $sqlstats = uc( getResp( "Enable collectstmt instrumentation? (N/Y)", $sqlstats ) );
                if ( ( $sqlstats eq "Y" ) || ( $sqlstats eq "N" ) ) 
                    {
                        $specified = 1;
                    }
            }

        if ( !$dynamicsql ) { $dynamicsql = "N"; }
            $dynamicsql = uc( getResp( "Enable SNAPSHOT FOR DYNAMIC SQL? (N/Y)", $dynamicsql ) );

        if ( $monitorlogspan eq "" ) { $monitorlogspan = "Y"; }
            $monitorlogspan = uc( getResp( "Monitor log spanning? (N/Y)", $monitorlogspan ) );

        $stopevent = getResp("Enter comma separated stop events, e.g. ANR#####E,ANR9999D_##########", $stopevent );
        $traceflags = getResp( "Enter traceflags", $traceflags );

        if ( ( $traceflags ne "" ) && ( uc($traceflags) ne "NONE" ) ) {
            $disableflags = getResp( "Enter traceflags to disable", $disableflags );
            my $done = 0;
            do {
                $maxsize = max(getIntResp( "Enter size limit for the tracefile. (1-65535MB, -1 unlimited)", $maxsize ), -1);
                if (   ( ( $maxsize > 0 ) && ( $maxsize < 65536 ) )
                    || ( $maxsize == -1 ) )
                {
                    $done = 1;
                }
            } until $done;
        }

        $specified = 0;

        while ( !($specified) ) 
            {
                if ( !$ffdccopy ) 
                    { 
                        $ffdccopy = "N"; 
                    }

                $ffdccopy = uc( getResp( "Copy FFDCLOG file? (N/Y)", $ffdccopy ) );

                if ( ( $ffdccopy eq "Y" ) || ( $ffdccopy eq "N" ) ) 
                    {
                        $specified = 1;
                    }
            }

        $cycleTime = ( $iterations - 1 ) * $sleeptime;
        #    $oneDay    = ( 3600 * 24 ) / $cycleTime;

    }
    else    
        {
            if (   ( !( $^O eq "MSWin32" ) && !( $^O eq "Windows_NT" ) ) && ( !( $showthreads eq "Y" || $showthreads eq "N" ) ) )
                {
                    die "\$showthreads must be either \"Y\" or \"N\"";
                }

            print "\nRunning with the following values configured in the script:\n";
            print "Server       : $server\n";

            if ( $^O eq "MSWin32" ) 
                {
                    print "tcpport      : $tcpport\n";
                }

            print "Administrator       : $administrator\n";
            print "Password            : " . "*" x ( length($password) ) . "\n";

            # db2instance check removed db2ilist

            #print "PMR                 : $pmr\n";

            if ( !( $^O eq "MSWin32" ) && !( $^O eq "Windows_NT" ) ) 
                {
                        print "Collect SHOW THREADS: $showthreads\n";
                }

            print "Collect SHOW DEDUPDELETE: $showdedupdelete\n";
            print "Collect SHOW ALLOC/MEM/MEMTREND: $showmemory\n";
            print "Collect SHOW REPLICATION: $showreplication\n";

            if ( ( $traceflags ne "" ) && ( uc($traceflags) ne "NONE" ) ) 
                {
                        print "Traceflags          : \"$traceflags\"\n";
                        print "Disable traceflags  : \"$disableflags\"\n";
                        print "Maxsize             : \"$maxsize\"\n";
                }

            print "Copy FFDCLOG file: $ffdccopy\n";
    }    
    

#
#   Check if the user running ServerMon is the supplied DB2 user
#

    if ( $prompt && ( ( $thisOperatingSystem eq "MSWin32" ) || ( $thisOperatingSystem eq "Windows_NT" ) ) ) 
        {
            $db2User        = getResp( "Enter the DB2 instance ID", $db2User );
            $db2Password    = readPassword( "Enter the DB2 instance password",  $db2Password );
        }    

#
# Set the SP Paths up before we start
#

SetSpectrumProtectPaths() ;
#
#   Make sure that we can access Spectrum Protect using the dsmadmc command
#

testSpectrumProtectAdminConnection() ;

#
#   Are we running as a Storage Agent
#

$storageagent = is_sta();

#
#	If the inbuilt ServerMon is running, we'll stop this execution
#

checkInbuiltServerMonStatus() ;


#
#   Before we start check that the user executing ServerMon is the passed DB2User or that we're running on a storage agent
#   If not the utility will exit
#

if($db2User ne $username)
    {
        if($storageagent == 0)
            {
                LogToBoth("\n\nYou are not running as the DB2 user, as specified in the ServerMon \"servermon.ini\" file parameter \"db2user\" or supplied via the console.\nThe Utility will now exit\n") ;
                exit 99 ;
            } 
    }


#
# Build reporting folder
#
  
prepareDocdir();

#
#   Build ServerMon spash header
#

WriteServerMonHeader() ;	

#
#   Before we start check that Spectrum Protect is available
#

check_tsm_connection();

#
#   Check if there are any existing ZIP files that we need to archive
#

list_runtime_folder();

#
#
#

# Prompt the user for information

print "This script starts IBM Spectrum Protect server instrumentation, when cancelling this script you may wish to stop server instrumentation via the IBM Spectrum Protect command 'instrument end' such that it doesn't continue to run indefinitely.\n\n";

my $rundir  = $showprefix = "";
my $db2      	= 0;
my @out;
my $stopquery1 = $stopquery2 = $where9999 = $wheremsgno = "";

if ( $delimeter ne "comma" || $delimeter ne "tab" ) 
    {
    	$delimeter = "comma" ;
    }

# compile the queries to check if a stop condition is met
if ( ( $stopevent ne "" ) && ( uc($stopevent) ne "NONE" ) ) {

    $stopevent = uc($stopevent);
    $stopevent =~ tr/ //ds;

    if (!$prompt) {
        print "Stop condition set  : \"" . $stopevent . "\".\n";
    }

    my @events    = split( ',', $stopevent );
    my $separator = "";
    my $andor     = " and ( ";

    foreach my $event (@events) {

        if ( index( $event, "ANR9999D_" ) == 0 ) {
            $where9999 .= $andor . "( message like \'$event%\' ) ";
            $andor = " or ";
        }
        else {
            $event =~ tr/A-Za-z //ds;
            $event += 0;
            $wheremsgno .= $separator . $event;
            $separator = ",";
        }
    }

    if ( $wheremsgno ne "" ) {
        $stopquery1 = "select message from actlog where originator='SERVER' and msgno in ( $wheremsgno )";
    }
    if ( $where9999 ne "" ) {
        $stopquery2 = "select message from actlog where originator='SERVER' and msgno in ( 9999 ) $where9999 )";
    }

}

print "\nDSM_DIR: " . $tsmpath . "\nDSM_CONFIG: " . $dsm_opt . "\n\n";

# enable commands if not in a STA environment 
# and prepare DB2 data collection

if ( !($storageagent) ) 
    {
        if ( hasSDpools() ) 
            {
                push( @extrasrv_commands, @container_pools) ;
            }

        push( @commands,        @more_commands );
        push( @extra1_commands, @extrasrv_commands );

        if ( defined( $ENV{"DB2INSTANCE"} ) && $instance eq "" ) 
            {
                $instance = $ENV{"DB2INSTANCE"};
            }

        while ( !($instancefound) ) 
            {
               if ( $prompt ) 
                    {
                        MessagesLogHandler(1) ;
                        $instance = getResp( "\nEnter the DB2 instance name", $instance );
                        MessagesLogHandler(2) ;
                    }

                if ( $instance ne "" ) 
                    {
                        Check_DB2() ;
                    }
            }

        #
        #   Before we start check that the user executing ServerMon is the passed DB2User or that we're running on a storage agent
        #   If not the utility will exit
        #

        LogToBoth("About to connect to database\n");

        ( $responce, $connectRC ) = run_database_command("connect to $dbalias");

        LogToBoth("@$responce") ;

        LogToBoth("Connecting to the TSM database gave return code : $connectRC\n") ;

        foreach $line (@out) {
            if ( $line =~ /^DB21018E/ || $line =~ /^DB21061E/ ) {
                exit;
            }
        }

        if ( !$connectRC ) {    # connect went OK

            $db2 = 1;

            if ( ( $^O eq "MSWin32" ) || ( $^O eq "Windows_NT" ) ) 
                    {
                        $statsiterations 	= max( 1, int( ( $iterations - 1 ) * $sleeptime / $statsinterval ) );
                        $statscmd 		= "db2 connect to $dbalias|db2pd -iostat t l $statsinterval $statsiterations %computername%";        	
                        $vmstatcmd   	= "db2pd -vmstat T $statsinterval $statsiterations %computername%" ;
                    }
        }
        else {
            LogToBoth("Error $connectRC connecting to $dbalias, DB information will not be collected.\n") ;
            $db2 = 0;
        }
    }
    else {
        LogToBoth("\nStorage agent environment\n") ;
        $db2     = 0;
        $summary = 0;
    }

    if ( $prompt ) 
        {
            my $delay = 10;

            LogToBoth("\nScript will start executing after " . $delay . "s delay.\n");
            for ( $delaycount = 0 ; $delaycount < $delay ; $delaycount++ ) {
                print ".";
                sleep(1);
        }

        print "\n";
    }

$SIG{INT} = 'stopScript';    #set control-C routine
print "\n";

#my $summaryname = $trccmdname = $trcname = $begints = $now = $csvcycle = $csvdiag = "";
my $traceactive = 0;
my $instractive = 0;
my $spanmax     = $filesChecked = $spanmaxAppId = 0;
my $spanmaxFile = "";
my $cyclefile 	= $intervalfile = "" ;

#
#	Before we start try and close any exisiting instrumentation from running
#

#prepareDocdir();

$prefix = strftime( "%Y%m%d-%H%M", localtime() ) . "-" . sprintf( "%08d", "0" );

get_instrumentation("$prefix-initial-inst-close-attempt.txt");

my $runCSVonce = 0;

RUNLOOP:
    my $firstcycle  = "true" ;
    my $serverpid   = 0 ;
    my $cyclestart  = time();

    $startsCompleted++;

    if($AlwaysOn eq "N")
        {
            my ( $StartDiagCycleTime, $ScheduledDiagCycleEndTime ) ;

            $StartDiagCycleTime           =  time() ;
            $ScheduledDiagCycleEndTime    =  time() + ( $Duration * 3600 ) ;
            
            LogToBoth("ServerMon is scheduled to complete at " . strftime("%m/%d/%Y %H:%M:%S",localtime($ScheduledDiagCycleEndTime)) . "\n") ;
            
            ProcessEndOfSection($now . "-00000000-oscmd-cycle.txt", @oscmd_cycle);

            while(time() <= $ScheduledDiagCycleEndTime )
                {
                    MainProcessingSubroutine() ;
                }
            
#            $timeNow = time() ;
            LogToBoth("The time now is " . strftime("%m/%d/%Y %H:%M:%S",localtime(time())) . ", and the scheduled finish time was " . strftime("%m/%d/%Y %H:%M:%S",localtime($ScheduledDiagCycleEndTime)) . "\n") ;
 
            $stopping = "has completed and will now stop after reaching time control" ;            
        }
        
    if($AlwaysOn eq "Y")
        {
            my $CollectionCycleEndTime  =   timelocal("00", "00", "00", (localtime)[3,4,5]) + ConvertToSeconds($CollCycleEndTime) ;
            
            if($CollectionCycleEndTime<time())
                {
                    $CollectionCycleEndTime  =  timelocal("00", "00", "00", (localtime)[3,4,5]) + ConvertToSeconds($CollCycleEndTime) + 86400 ; 
                }
            
            LogToBoth(".. The first collection cycle will end at approximately " . strftime("%m/%d/%Y %H:%M:%S",localtime($CollectionCycleEndTime)) . " and then approximately every 24hrs after\n") ;
            
            while(1)
                {          
                    LogToBoth(".. This collection cycle will end at approximately " . strftime("%m/%d/%Y %H:%M:%S",localtime($CollectionCycleEndTime)) . ", once the current diagnostic cycle completes\n\n") ;
                    
                    ProcessEndOfSection($now . "-00000000-oscmd-cycle.txt", @oscmd_cycle);
                    
                    $firstcycle  = "true" ;
                    
                    while(time() <= $CollectionCycleEndTime)
                        {
                            MainProcessingSubroutine() ;
                        }  

                    $stopping = "will now finalize cycle\n" ;
                    finalizeCycle() ;
                    
                    $CollectionCycleEndTime = $CollectionCycleEndTime + 86400 ;

                    prepareDocdir();                                                
                }
        }
        
    finalizeCycle() ;

    LogToBoth("\n\n.. and done.\n") ;

    exit(0);

# Subroutines:

sub MainProcessingSubroutine
{
    if(!$prompt)
        {
            load_command_variables();
        }

        $now     	= strftime( "%Y%m%d-%H%M", localtime() );
        $showprefix 	= $prefix = "$now-" . sprintf( "%08d", ++$cyclesCompleted );
        
        CopyCommandsIniFile() ;

        if ( $firstcycle eq "true" ) 
        	{
                    Debug_Message("Starting BuildFirstCycleReports") ;
                    buildFirstCycleReports() ;
	            $firstcycle = "false";
       	 	}
       	 	                
        $csvdiag  = "$showprefix-dsm-diag";
        $csvcycle = "$now-99999999-dsm-cycle";

        my $begin    = $now;
        my $slept    = 0;
        my $stopcond = 0;

        # get the server pid for procstack command
        if ( ( $serverpid == 0 ) && ( !$IT21957 ) ) 
        	{
            	$serverpid = &get_pid();
        	}
        
        # check_service_status();
        
        Debug_Message("Instr begin") ;

        if($instrumentation =~ /y/i)
            {
                `$cmd instr begin $collectstmt`;
            }

        $instractive = 1;
        
        Debug_Message("Running Stats Command : $statscmd") ;

        if ( $statscmd ne "" ) 
	 	{
                    Debug_Message("About to run command : $statscmd" ) ;
                    
                    if ( ( $^O eq "MSWin32" ) || ( $^O eq "Windows_NT" ) ) 
                            {
                                system( 1, "$statscmd > iostat-files/$prefix-iostat.txt &" );
                            }
                            else 
                                {
                                    system("$statscmd > iostat-files/$prefix-iostat.txt &");
                                }

                    print("-- Redirecting iostat to iostat-files/$prefix-iostat.txt\n") ; GeneralPrint() ;
        	}
        
        Debug_Message("In NFSStatsCmd : $nfsstatscmd") ;
         
        if ( $nfsstatscmd ne "" ) 
        	{
         		system( "$nfsstatscmd > $prefix-nfsiostat.txt &" );
        		LogToBoth("\nRedirecting nfsiostat to $prefix-nfsiostat.txt\n");
        	}
        	
        	
        # check_service_status();
        
        Debug_Message("Running VMStatCmd : $vmstatcmd") ;

        if ( $vmstatcmd ne "" ) 
        	{
                    if ( ( $^O eq "MSWin32" ) || ( $^O eq "Windows_NT" ) ) 
                        {
                            system(1, "$vmstatcmd > vmstat-files/$prefix-vmstat.txt &") ;	 	        		        	
                        }
                        else {
                                system("$vmstatcmd > vmstat-files/$prefix-vmstat.txt &");
                             }

                    print("\n-- Redirecting vmstat to vmstat-files/$prefix-vmstat.txt\n") ; GeneralPrint() ;
        	}

        # check_service_status();
        
        Debug_Message("In LPARStat : $lparstatcmd") ;

        if ( ( $^O eq "aix" ) && ( $lparstatcmd ne "" ) ) 
            {
                system("$lparstatcmd > lparstat-files/$prefix-lparstat.txt &");
                LogToBoth("\n-- Redirecting lparstat to $prefix-lparstat.txt\n") ; GeneralPrint() ;
            }

        $i = 0;

        # check_service_status();    # Added MW 0603

        $showprefix = "$now-" . sprintf( "%08d", $cyclesCompleted );
        $showoutfile = "show-files/$showprefix-show.txt";

        # open (SHOWOUT, ">$showprefix-show.txt") || die "Can't open SHOWOUT for writing\n";

        Debug_Message("Iterations  $i : $iterations") ;
        
        while ( $i != $iterations ) 
            {

            # first check if the stop condition is met
            # check_service_status();    # Added MW 0603

            my $union = $stopquery = "";

            if ( $stopquery1 ne "" ) {
                $stopquery = $stopquery1 . " and date_time >= \'$begints\'";
                $union     = " union ";
            }

            if ( $stopquery2 ne "" ) {
                $stopquery .=
                  $union . $stopquery2 . " and date_time >= \'$begints\'";
            }

            if ( $stopquery ne "" ) {
                @out = `$cmd \"$stopquery\"`;

                foreach $line (@out) {
                    @conditions = split( /\,/, $stopevent );

                    foreach $condition (@conditions) {
                        if ( $line =~ /^$condition/ ) {
                            ( $condmsg = $line ) =~ s/\s+$//;
                            $stopcond = 1;
                        }
                    }
                }
            }

            # check_service_status();
            
            foreach $command (@commands) 
            	{
                    Debug_Message("Running \@commands : $command") ;
                    `$cmd  \" $command \" >> $showoutfile`;
                    Debug_Message("  Completed : $command") ;
                    GeneralPrint() ;
                }

            # check_service_status();
            
            Debug_Message("In ProcStack") ;

            if ( $ProcStack eq "Y" ) {
                
                print("ProStack was selected\n") ; GeneralPrint() ;
                
                if ( defined $procstackbin ) 
                    {
                        if ( $^O eq "aix" ) 
                            {
                                eval {
                                        local $SIG{ALRM} = sub { die "alarm\n" };
                                        my $now = strftime( "%H:%M:%S", localtime() );
                                        alarm $timeout;
                                        `$procstackbin $serverpid >> $showoutfile`;
                                        alarm 0;
                                        Debug_Message("$procstackbin $serverpid >> $showoutfile") ;
                                        print(" - completed, reset timeout: 0\n") ;
                                     };
                            }
                            else {
                                    Debug_Message("$procstackbin $serverpid >> $showoutfile") ;
                                    `$procstackbin $serverpid >> $showoutfile`;
                                }
                    }

                if ($@) {
                    die unless $@ eq "alarm\n";

                    if ( defined $procrun ) 
                        {
                            `$procrun $serverpid`;
                        }

                    open( SHOWOUT, ">>$showoutfile" ) || die "Can't open $showoutfile for writing\n";
                    print SHOWOUT "\n\n $procstackbin $serverpid timed out after $timeout seconds.\n";
                    close(SHOWOUT);

                }
                else {
                    # nothing to do here
                }

                if ( $#out < 0 ) 
                    {
                        print "Reset of server pid\n";
                        $serverpid = 0;
                    }
            }

            # close(SHOWOUT);
            print "-- Wrote $showoutfile\n";

            # check_service_status();    # Added MW 0703

            if ($stopcond) {
                LogToBoth("Stop condition detected: $condmsg\n") ;
                $stopping = "stopping";
                stopScript();
            }

            $i++;
            
            Debug_Message("Starting Iterations") ;

            if ( $i < $iterations ) {
                for ( $count = 1 ; $count <= $sleeptime ; $count++ )
                {                      # to handle interrupt on Windows
                    sleep(1);

                    if ( $count % 5 == 0 ) {
                        # check_service_status();
                    }
                }

                $slept          += $sleeptime;
                $now 		    = strftime( "%Y%m%d-%H%M", localtime() );
                $showprefix 	= "$now-" . sprintf( "%08d", $cyclesCompleted );
                $showoutfile 	= "show-files/$showprefix-show.txt";

				# open (SHOWOUT, ">$showprefix-show.txt") || die "Can't open SHOWOUT for writing\n";
            }            
        }

        get_instrumentation("$prefix-instr.txt");

        my $db2pdFile = "";

        # check_service_status();    

        if ($db2) 
            {
                open( DB2OUT, ">db2-files/$prefix-db2.txt" )
                  || die "Can't open DB2OUT for writing\n";

                foreach $db2command (@db2commands) 
                    {
                        Debug_Message("Running db2command : $db2command") ;
                        print DB2OUT "COMMAND (" . getTimeStamp() . ") : db2 " . $db2command . "\n\n";
                        ( $responce, $rc ) = run_database_command($db2command);
                        print DB2OUT "@$responce" ;
                        GeneralPrint() ;
                        Debug_Message("  Completed : $db2command") ;
                        }

                print "-- Wrote db2-files/$prefix-db2.txt\n";
                close(DB2OUT);

                # check_service_status();    #	Added MW 0603

                $db2pdFile = "db2pd-files/$prefix" . "-db2pd.txt";

                open( DB2PDOUT, ">$db2pdFile" ) || die "Can't open DB2PDOUT for writing\n";

                foreach $db2pdcommand (@db2pdcommands) 
                    {
                        Debug_Message("Running db2pdcommand : $db2pdcommand") ;
                        @out = `db2pd $db2pdcommand 2>&1`;
                        print DB2PDOUT "COMMAND (" . getTimeStamp() . ") : db2pd " . $db2pdcommand . "\n\n";
                        print DB2PDOUT "@out\n\n";
                        GeneralPrint() ;
                        Debug_Message("  Completed : $db2pdcommand") ;
                    }

                print "-- Wrote $db2pdFile\n";
                close(DB2PDOUT);

                # check_service_status();          
            }

        Debug_Message("Starting correlateTxnLog") ;

        if ( ( $monitorlogspan eq "Y" ) && ( -e $db2pdFile ) ) 
        	{
                    correlateTxnLog($db2pdFile);
        	}

        if ( $actlog || $summary ) 
        	{
                    get_actlog_summary();
        	}

        $longsleep = $repeat - $slept;
        
        Debug_Message("Starting continious") ;
        if ( !$continuous ) 
        	{
                    for ( $count = 1 ; $count <= $longsleep ; $count++ )
                            {    # to handle interrupt on Windows
                                    sleep(1);
                                    # check_service_status();
                            }
        	}

        if ( ( $^O eq "MSWin32" ) || ( $^O eq "Windows_NT" ) ) 
        	{
	            # wait for spawned child processes
	            my $kid;
	            
	            do {
                            $kid = waitpid( -1, WNOHANG ) ;
                        } while $kid > 0;
        	}

        my $fileCounter     = 1;
        my $buildCSVcommand = "";
        
        chdir("csv-files") ;
        
        Debug_Message("Starting csvcommand") ;

        foreach $csvcommand (@csv_diag) 
        	{
                    Debug_Message("$cmdCsv \"$csvcommand\"", "$csvdiag-" . $fileCounter . ".csv") ;
                    runCommand("$cmdCsv \"$csvcommand\"", "$csvdiag-" . $fileCounter . ".csv", $csvcommand  ) ;	
	            $fileCounter++;
                    Debug_Message("  Completed : $cmdCsv $csvcommand") ;
        	}
        	
        chdir("..") ;

        ProcessEndOfSection($prefix . "-oscmd-diag.txt", @oscmd_diag);

        LogToBoth("\nCompleted diagnostic cycle " . $cyclesCompleted . "\n") ;

        # check_service_status();   

        print "\n\n";
}

sub ConvertToSeconds
    {
        ($timeIN) = @_ ;        
        $inSeconds = ( (substr($timeIN,0,2) * 3600 ) + ( substr($timeIN,-2) * 60 )  ) ; 
        return $inSeconds ;
    }

sub CopyCommandsIniFile
    {
        my $CommandsIniFile         = "../commands.ini" ;
        my $TargetCommandsIniFile   = "ini-files/$showprefix-commands.ini" ;
        
        if ( copy( $CommandsIniFile, $TargetCommandsIniFile ) ) 
            {
                print "\n-- Copied commands.ini to $TargetCommandsIniFile.\n";
            }            
    }

sub ProcessEndOfSection	
	{	
        Debug_Message("Starting ProcessEndOfSection") ;
		my ( $outputFile, @CommandsToProcess )	= @_ ;
					
		my %SupportedSystems = (
									'dos' 		=>	'WIN'	,
									'MSWin32'	=>	'WIN'	,
									'aix'		=>	'AIX'	,
									'linux'		=>	'LIN'	,
							    ) ;
	
		foreach $cycleCommand ( @CommandsToProcess )
                        {
                                ($system, $thisCycleCommand) = split(":", $cycleCommand) ;
                                
                                Debug_Message("Running thisCycleCommand : $thisCycleCommand") ;

                                if ( exists $SupportedSystems{$thisOperatingSystem} )
                                        {

                                                if ($SupportedSystems{$thisOperatingSystem} eq "WIN" && $system eq "WIN")
                                                        {
                                                                runCommand($thisCycleCommand, $outputFile, "") ;
                                                        }

                                                if ($SupportedSystems{$thisOperatingSystem} eq "LIN" && $system eq "LIN")
                                                        {
                                                                runCommand($thisCycleCommand, $outputFile, "") ;   
                                                        }

                                                if ($SupportedSystems{$thisOperatingSystem}  eq "AIX" && $system eq "AIX")
                                                        {
                                                                runCommand($thisCycleCommand, $outputFile, "") ;
                                                        }
                                        }
                        }
			
		print "-- Wrote $outputFile\n" ;
	}

sub buildFirstCycleReports
{
    Debug_Message("Starting buildFirstCycleReports") ;
    
    my $systemfile 		= "$now-00000000-system.txt";
    my $configfile 		= "$now-00000000-config.txt";
    my $volhfile   		= "$now-00000000-volhistsize.txt";
    my $extrafile  		= "$now-00000000-extra.txt";
    $intervalfile		= "$now-00000000-oscmd-diag.txt";
    $cyclefile			= "$now-00000000-cycle.txt";

    my $rootFile = findRootCommandFiles() ;

    if ( $rootFile ne "" ) 
        {
            copy( "$startdir$rootFile", "." ) ;
        }

    # check_service_status();
    
    `$cmd  QUERY SYSTEM > $systemfile`;
    print "-- Wrote $systemfile\n";

    # check_service_status();

    if ( !($storageagent) ) {
       `$cmd  SHOW CONFIG > $configfile`;
    }

    print "-- Wrote $configfile\n";

    foreach $command (@extra1_commands) 
        {
                Debug_Message("Starting \@extra1_commands : $command") ;
                `$cmd  \" $command \" >> $extrafile`;
                # check_service_status();
                GeneralPrint() ;
                Debug_Message("  Completed : $command") ;
        }

    print "-- Wrote $extrafile\n";

    `$cmd \"select count(*) as volumes_in_volhist from volhistory\" > $volhfile`;
    print "-- Wrote $volhfile\n"; GeneralPrint() ;

    # check_service_status();

    if ( ( $traceflags ne "" ) && ( uc($traceflags) ne "NONE" ) ) 
        {
                $traceactive = startTrace();
        }
}

sub runCommand
    {        
        my ( $command, $FileOut, $csvCommand)  = @_ ;
        Debug_Message("In runCommand : $command -> $FileOut") ;
		
        open(ReportOut, ">>$FileOut") or die "Can't open $out for writing\n" ;
				
        @out = `$command` ;
        
        if($csvCommand eq '')
            {
                print ReportOut printHeader($command);
            } else
            {
                print ReportOut printHeader($csvCommand);
            }

        foreach $line ( @out ) 
            {
                    print ReportOut "$line" ;
            }

        close(ReportOut) ;

        if (index($FileOut, "99999999") == -1)
            {
                    print "-- Wrote $FileOut \n" ;
            }
            
        GeneralPrint() ;
    }
        
sub printHeader
    {
        ( my $command ) = @_ ;
        (my $newCommand = $command) =~ s/\Q$password\E/********/g ;
        
        my $header          = "COMMAND (" . getTimeStamp() . "): " . $newCommand ;
        my $headerLength    = length($header)+4 ;
        return "\n" . "*" x $headerLength . "\n* $header *\n" .	"*" x $headerLength . "\n";
    }
		
sub run_database_command {
    Debug_Message("In run_database_command") ;
    $thisCommand = shift(@_);

    if ( $thisCommand =~ /^select/i ) {
        $thisCommand =~ tr/\r\n//d;

        if (   ( $thisOperatingSystem eq "MSWin32" ) || ( $thisOperatingSystem eq "Windows_NT" ) )
	        {
	            $thisCommand =~ s/>/\ "\">\"\" /g;
	            $thisCommand =~ s/</\ "\"< \"\" /g;	
		        $thisCommand = "db2cmd -c -i \"db2 connect to $dbalias user $db2User using '$db2Password' && db2 $thisCommand\"" ; 
	        }
        else {
            	$thisCommand = "db2 connect to $dbalias && db2  \"$thisCommand\"" ;
            	#$thisCommand = "db2 connect to $dbalias user tsmuser1 using tsmuser1 && db2  \"$thisCommand\"";
        	}
    }
    else {
        if (   ( $thisOperatingSystem eq "MSWin32" )
            || ( $thisOperatingSystem eq "Windows_NT" ) )
        {
            $thisCommand = "db2cmd -c -i \"db2 $thisCommand\"";
        }
        else {
            $thisCommand = "db2 \"$thisCommand\"";
        }
    }

    @out = `$thisCommand`;

    $returnCode = $?;

    return ( \@out, $returnCode );
}

sub load_command_variables {
    Debug_Message("load_command_variables") ;
    print "\nLoading runtime commands\n";

    #
    #	Build TSM commands into arrays from external files
    #

    build_commands("$startdir/commands.ini");

    if ( $showdedupdelete eq "Y" ) {
        push @more_commands, "SHOW DEDUPDELETE";
    }

    if ( $showmemory eq "Y" ) {
        push @more_commands, ("SHOW ALLOC", "SHOW MEM", "SHOW MEMTREND");
    }

    if ( $showreplication eq "Y" ) {
        push @more_commands, "SHOW REPLICATION";
    }

    if ( $dynamicsql eq "Y" ) {
        push @db2commands, "get snapshot for dynamic sql on $dbalias";
    }

    if ( $showthreads eq "Y" ) {
        push @commands, "SHOW THREADS";
    }

    if ( $hasSDpools_status == 1 ) 
        {
            push( @extrasrv_commands, @container_pools);
        }       

    if ( $storageagent == 0 ) {
        push( @commands,        @more_commands );
        push( @extra1_commands, @extrasrv_commands );
    }
    
    #
    #   If we're on Linux set ProcStack to "Y"
    #
    
    if ( $thisOperatingSystem eq "linux" )
        {
            Debug_Message("Assigning \$ProStack from $ProcStack to \"Y\" ") ;
            $ProcStack = "Y" ;
        }

    #
    #	Make the TSM database alias uppercase
    #

    $dbalias = uc($dbalias);

#
#	The following two arrays contain references to variables passed from ini files. Loop through them and replace with correct values
#

    for ( @db2extracommands, @db2pdcommands ) { s/\$dbalias/$dbalias/g }
    for ( @db2commands,      @db2commands )   { s/\$dbalias/$dbalias/g }

}

sub check_service_status 
    {
        Debug_Message("In check_service_status") ;
        if ( ( $^O eq "MSWin32" ) and ( $WindowsService == 1 ) ) 
        	{
            	if ( SERVICE_RUNNING != Win32::Daemon::State() ) 
            		{
    		            finalizeCycle();
    		            last RUNLOOP;      		}
        	}

        return 1;
    }

sub check_tsm_connection {

    $optfile = "$startdir$optfile";
    
    Debug_Message("The options file is set to : $optfile") ;

    if ( ( -e $optfile ) && ( $optfile ne "" ) ) 
    	{
	        print "INFO: Option file (\"$optfile\") found, ";
	
	        @out = `dsmc Q SE -optfile=$optfile`;
	        $rc  = $?;
	
	        if ( !$rc ) 
	        	{
	            	LogToBoth("connected to server: will try to archive collected docs.\n\n");
	            	$tsm_available = 1;
	        	}
	        else {
	            	print("unable to connect to server (RC=$rc): will not archive collected docs.\n\n");
	        	}
	
	        print @out;

    	}
    elsif ( $optfile eq "" ) {

        # nothing to do...

    }
    else {
        LogToBoth("INFO: Option file (\"$optfile\") not found, will not archive collected docs.\n") ;
    }
}

sub read_ini_files {

    local ($basefolder, $ini_file_name) = @_;
    local ( $variable_comment, $variable_name, $variable_value, $create_variable );

    open my $ini_file, '<', "$basefolder$ini_file_name" or die "\n\nError:Can't open $basefolder$ini_file_name\n";

    while ( my $line = <$ini_file> ) {
        chomp $line;
        if ($line =~ /^(\w+)\s*=\s*([^#]*)/)
			    {
			    		$variable_name		=		$1 ;
			    		$variable_value		=		$2 ;
			    		$variable_value		=~ 		s/\s+$//;
			    		$variable_comment	=		$' ;

#
#	Check to see if the passed variable value is a string or a numeric and set corectly
#

            if ( $variable_value !~ /^-?[0-9]*$/gm ) {

                $variable_value = "N" if ( $variable_value eq '#' );
                $variable_value = "Y" if ( $variable_value eq 'y' );
                $variable_value = "N" if ( $variable_value eq 'n' );

                $create_variable = "\$$variable_name = \"$variable_value\" ;";
            }
            else {
                $create_variable = "\$$variable_name = $variable_value ;";
            }

            #
            #	Dynamically create the new variable
            #

            eval $create_variable;

            next;
        }
    }
}

# Create array for commands stored in external file
sub build_commands {
    Debug_Message("In build_commands") ;
    my ( $newCommand, $new_array, $lineIn, $a1 ) ;
    my $MultiLineCommand  = 0;
    my ($file_name)      = @_;
    my @tmpCommands      ;
    my $first_pass       = 1 ;
#    my $new_array = " " ;

    open( FILE, "<", $file_name ) or die("Can't open file : $file_name $!");

    my @tmp = <FILE>;

    foreach $lineIn (@tmp) {

        $lineIn =~ s/\015?\012?$//;
        $lineIn =~ s/\t+/ /g;
        $lineIn =~ s/^\s+|\s+$//g;
        
        #
        #   New Header section
        #

        if ( $lineIn =~ /^\[(.*)\]/ ) 
            {
                if ($first_pass) 
                    {
                        $new_array  = $1;
                        $first_pass = 0;
                        next;
                    } else {
                                $a1 = "\@$new_array = \@tmpCommands ;";
                                eval $a1;
                                $new_array = $1;
                                undef(@tmpCommands);
                                next;
                            }
            }

        #
        #	Single Line Commands
        #
        
        #Check for comment line
        if ($lineIn =~ /^#/)
            {
                next ;
            }

        if ( $MultiLineCommand == 0 && $lineIn =~ /^"/ && $lineIn =~ /"$/ ) 
            {
                $newCommand = $lineIn;
                $newCommand =~ s/"//g;

                if ( $lineIn ne "" ) 
                    {
                        if ( $newCommand !~ /^#/ ) 
                            {
                                push( @tmpCommands, $newCommand );
                            }

                        next;
                    }
            }

        if ( $MultiLineCommand == 0 && $lineIn !~ /^"/ ) 
            {
                $newCommand = $lineIn;

                if ( $newCommand ne "" ) 
                    {
                        if ( $newCommand !~ /^#/ ) 
                            {
                                push( @tmpCommands, $newCommand );
                            }

                        next;
                    }

            }

        #
        #	Multiline Support
        #

        if ( $lineIn =~ /^"/ ) 
            {
                $MultiLineCommand = 1;
                $newCommand       = $lineIn;
                next;
            }

        if ( $MultiLineCommand == 1 && $lineIn !~ /"$/ ) 
            {
                $newCommand = join( ' ', $newCommand, $lineIn );
                next;
            }

        if ( $MultiLineCommand == 1 && $lineIn =~ /"$/ ) {

            if ( $newCommand !~ /^#/ ) 
                {
                    $newCommand = join( ' ', $newCommand, $lineIn );
                    $MultiLineCommand = 0;
                    $newCommand =~ s/^.(.*).$/$1/;
                    $newCommand =~ s/"/\\"/g;
                    $newCommand =~ s/\t+/ /g;

                    push( @tmpCommands, $newCommand );
                }

            next;
        }
    }

    $a1 = "\@$new_array = \@tmpCommands ;";
    eval $a1;

    close(FILE);
    
    LogToBoth("\nLeaving build_commands") ;
    
}

# Get a user response and return it
sub getResp {
    local ( $promptText, $default ) = @_;

    print "$promptText [$default]:";
    $| = 1;
    $_ = <STDIN>;
    chomp;
    return $_ ? $_ : $default;
}

# Get a numeric user response and return it
sub getIntResp {
    local ( $promptText, $default ) = @_;

    print "$promptText [$default]:";

    do {

        $return = 0;
        $|      = 1;
        my $value = <STDIN>;

        chomp($value);

        if ( $value =~ m/[^0-9\-]/ ) 
        	{
            	print "Bad number: \"$value\" - $promptText [$default]:";
        	}
        else {

            if ( length($value) == '0' ) {
                return $default;

            }
            elsif ( $value < -1 ) {
                print "Reset given value $value to -1 (indefinite).\n";
                return -1;

            }
            else {
                return $value;
            }

            $return = 1;
        }

    } until $return;

}

# Read a password from the command line
# Tries to read the password hidden first
# Reads the password in clear text if hidden reads fail
# Different hidden read styles to work on Windows/Unix
sub readPassword {
    local ( $promptText, $default ) = @_;

    my $pw = undef();

    # Windows style
    eval {
        require Term::ReadKey;

        Term::ReadKey::ReadMode('noecho');
        print "$promptText ["
          . "*" x ( length($default) )
          . "] (no echo to the screen):";
        chomp( $pw = <STDIN> );
        Term::ReadKey::ReadMode('restore');
    };

    # Unix style
    if ( !defined($pw) ) {
        eval {
                print "$prompt ["
              . "*" x ( length($default) )
              . "] (no echo to the screen):";
            system( 'stty', '-echo' );
            chop( $pw = <STDIN> );
            system( 'stty', 'echo' );
        };
    }

    if ( ( defined($pw) ) && ( !length($pw) ) ) {
        $pw = $default;
    }
    elsif ( defined($pw) ) {
        print "*" x length($pw);
    }

    # read the password visible if we were not able to read it hidden
    if ( !defined($pw) ) {
        print "$prompt ["
          . "*" x ( length($default) )
          . "] (warning, it will echo to the screen):";
        chop( $pw = <STDIN> );
    }

    print "\n";

    return $pw;
}

# catch CTRL-C and stop the script...
sub stopScript 
    {
        finalizeCycle();
        print ".. and done.\n\n";
        exit;
    }

# Stop instrumentation and tracing  on the server
# and do final doc completion tasks..

sub finalizeCycle {
    my $fileCounter     = 1;
    my $buildCSVcommand = "";

    ProcessEndOfSection($now . "-99999999-oscmd-cycle.txt", @oscmd_cycle);
    
    chdir("csv-files") ;
     
    foreach $csvcommand (@csv_cycle) 
    	{       
            runCommand("$cmdCsv \"$csvcommand\"", "$csvcycle-" . $fileCounter . ".csv", $csvcommand  ) ;       
            $fileCounter++;
    	}

    chdir("..") ;

    print "\n";

    if ($instractive) 
        {
            LogToBoth("\nScript $stopping, please wait while stopping instrumentation on the server..\n") ;
            get_instrumentation("$now-99999999-lastinstr.txt");
        }
        else {
                LogToBoth("\nScript $stopping\n") ;
            }

    if ($traceactive) {
        print "\n.. stop active trace ..\n";
        stopTrace();
    }
    
    print "\n.. collecting transferstats ..\n";

# open (POOLS,"$cmd select STGPOOL_NAME from stgpools |") || die "Can't open $cmd - $!";
    my @poollist = `$cmd -dataonly=yes select STGPOOL_NAME from stgpools`;
    $tansferstatsfile = "$now-99999999-transferstats.txt";
    $extrafile        = "$now-99999999-extra.txt";

    $i = 2;

    # while ($pool = <POOLS>) {
    foreach $pool (@poollist) {

        $pool =~ s/\s+$//;    # remove trailing whitespaces

        if ( $pool =~ m/^ANS8002I/ ) {
            print "-- Wrote $tansferstatsfile\n";
#            close(POOLS);
        }
        elsif ( ( $i > 13 ) && ( length($pool) ) ) {
            print "-- [" . $pool . "]\n";
            @out = `$cmd SHOW TRANSFERSTATS $pool >> $tansferstatsfile`;
        }

        $i++;
    }

    # $summary indicates if this is STA or not
    if ($summary) {

        open( DB2OUT, ">$extrafile" ) || die "Can't open DB2OUT for writing\n";

        foreach $command (@extra2_commands) {
            @out = `$cmd  \"$command\"`;
            print DB2OUT "@out";
        }

        if ($db2) {

            foreach $db2command (@db2extracommands) # process extra DB2 commands
            {
                print DB2OUT "COMMAND (" . getTimeStamp() . ") : $db2command\n\n";
                ( $responce, $rc ) = run_database_command($db2command);
                print DB2OUT "@$responce";
            }

            print "-- Wrote $extrafile\n";
            close(DB2OUT);

        }

        #print "\n-- Wrote $extrafile\n";
    }

    if ( $actlog || $summary ) {
        print "\n.. Collecting actlog and/or summary information ..\n"; GeneralPrint() ;
        get_actlog_summary();
    }

    if ( $^O eq "aix" ) {

        if ( $lparstatcmdfinal ne "" ) {
            print "\n.. collecting $lparstatcmdfinal ..\n"; GeneralPrint() ;
            `$lparstatcmdfinal > lparstat-files/$now-99999999-lparstat.txt &`;
            print "-- Wrote $now-99999999-lparstat.txt\n";
        }

        
    }
	
	#
	#	Identify instance directory
	#
    
			my $insthome 	= "";
            my $logfile  	= "";
			my $copytarget	= "" ;
			my @filesToCopy  ;

            if (   ( $^O eq "MSWin32" )  || ( $^O eq "Windows_NT" )  || ( $^O eq "cygwin" ) )
                {
                    my $registry = "HKEY_LOCAL_MACHINE/SOFTWARE/IBM/ADSM/CurrentVersion/Server/$ENV{'DB2INSTANCE'}" ;
                    my $regKey  = new Win32::TieRegistry $registry, { Access => KEY_READ(), Delimiter => "/" } ;
                    $insthome   = $regKey->GetValue("Path") . "\\";
                }
                else {
                        $insthome = $ENV{"DSMI_LOG"};
                        #$insthome =~ s/sqllib//;
                    }
					
	
    if ( $ffdccopy eq "Y" ) 
        {		

			@out = `$cmd -dataonly=yes q opt FFDCLOGNAME`;
			
			# Identify the FFDC log name
			foreach $line (@out) 
				{
					if ( $line =~ /^ *FFDCLogName +(\S+) *$/ ) 
						{
							$logfile    = "$1";
						}
				}

			@FilesToCopy  = glob "$insthome\\$logfile*" ;		
			$copytarget   = "ffdc-files/";

			foreach my $FileToCopy (@FilesToCopy)
				{					
					if ( copy( "$FileToCopy", $copytarget ) ) 
						{
							LogToBoth("-- Wrote $FileToCopy to $copytarget") ;
						}
				}    
          }
		  
	#
	#	Copy the servermon.ini filr
	#
	
		if ( copy( "../servermon.ini", "ini-files/$now-99999999-servermon.ini" ) ) 
			{
				LogToBoth("-- Completed copy of servermon configuration file") ;
			}

	#
	#	Attempt to copy protect server database backup API instrumenation file
	#	We assume the instrumentatio file is dsminstr.log and that it is in the instance directory
	#
			
	if ( copy( "$insthome/dsminstr.log", "$now-99999999-dsminstr.log" ) ) 
		{
			LogToBoth("-- Completed copy of server database backup API instrumenation file") ;
		}


	#
	#	Attempt to copy protect server options file
	#	We assume the options file ends with .opt and that it is in the instance directory
	#
			
	if ( copy( "$insthome/dsmserv.opt", "$now-99999999-dsmserv.opt" ) ) 
		{
			LogToBoth("-- Completed copy of server options file") ;
		}

    LogToBoth("\n.. doc preparation ..\n\n") ;
    
    chdir($startdir);

    $zip_file_to_archive = zip_report_files($targetdir);

    if ( $tsm_available == 1  )
            {

            if ( archive_zip_file($zip_file_to_archive) == 0 ) 
                    {
                            LogToBoth("\nZipping to $zip_file_to_archive complete.\n\n") ;
                            delete_target_dir($targetdir);
                    }
            }

    LogToBoth("..\n\n") ;

}

sub getDB2JavaBin() {
    my @getDB2 = `db2level`;

    if ( $? == -1 )    # db2level is not available
	    {
	        return "No_DB2";
	    }

    foreach (@getDB2) 
    	{
            $db2BaseFolder = $1 if ( $_ =~ /^Product is installed at "(.*)"/ );
    	}

    $javaFolder = ( $thisOperatingSystem eq "MSWin32" ? "" : "64" );

    return "$db2BaseFolder/java/jdk$javaFolder/bin/jar";
}

sub usePerlZIP {
    
    ($zip_folder) = @_;
    
    #
    #	Check if Perl's Archive:Zip is available
    #
    
    unless (eval "require Archive::Zip")
		{
		  $ArchiveStatus = 0 ;
		} else { 
					$ArchiveStatus = 1 ;
			   }

    LogToBoth(".. Creating zip archive $zip_folder\n") ;
    
    if ($ArchiveStatus)
    	{    		
    		Archive::Zip->import(qw( :ERROR_CODES ));
                my $zip = Archive::Zip->new();
                $zip->addTree('.');
		
		    unless ( $zip->writeToFileNamed( $zip_folder . ".zip" ) == AZ_OK() ) 
		    	{
		        	LogToBoth('\n\nWrite error creating zipfile.\nAll docs collected are available under $startdir/$targetdir\n\n');
                                die ;
		    	}
		    	
		    if ( $tsm_available == 0 ) 
		    	{
		        	LogToBoth(".. Documentation collection complete, please provide $zip_folder.zip.\n") ;
		    	}    		
    	} else  {
    				LogToBoth(".. No Zip package found, all docs collected are available under $startdir/$targetdir\n\n") ;
    				return 1 ;	#	Perl Zip not available
    			}
    			
    return 0 ;	# Perl Zip completed successfully
    
}

sub useDB2ZIP {
    my ($zip_folder) = @_;
    my $zipCommand   = "$db2ZipProgram -cMf $zip_folder.zip $zip_folder";
    
    Debug_Message("About to run command : $zipCommand") ;
    
    my @callZip      = `$zipCommand`;

    if ( $? == 0 )  
    	{
#        	chdir("$startdir");
#         	print ".. Deleting folder $zip_folder\n";
#        	rmtree( $zip_folder, 0 ) or warn "Couldn't remove folder $zip_folder\n";
            delete_target_dir($zip_folder) ;
    	}

    if ( $tsm_available == 0 ) 
    	{
            print ".. Documentation collection complete, please provide $zip_folder.zip.\n";
    	} 
}

sub zip_report_files 
{
    ($folder_to_zip) = @_;

    $db2ZipProgram = getDB2JavaBin();
        
        if ( $db2ZipProgram ne "No_DB2" ) 
            {
                Debug_Message("Calling DB2 zip to compress file : $folder_to_zip") ;
                useDB2ZIP($folder_to_zip);
            }
            else 
                {
                  Debug_Message("Calling ZIP to compress file : $folder_to_zip") ;  
                  usePerlZIP($folder_to_zip) ;
                }

    return $folder_to_zip . ".zip";
}

sub archive_zip_file {

    ($archive_file) = @_;

    my $endts = strftime( "%m/%d/%Y-%H:%M:00", localtime() );
    $archive_command = "dsmc archive $archive_file -su=y $deletefiles $archmc -desc=\"servermon: $begindate-$begintime to $endts\" -optfile=$optfile";
    Debug_Message("About to issue command : $archive_command") ;

    #	For Windows archive we need to switch the slashes around
    if (   ( $^O eq "MSWin32" ) || ( $^O eq "Windows_NT" ) || ( $^O eq "cygwin" ) )
	    {
	        $archive_command =~ s/\//\\/g;
	        $archive_command =~ s/\//\\/g;
	    }
	    
    print "\n..Submitting: \'$archive_command\'\n\n";

    @out = `$archive_command`;
    $rc  = $?;

    print "..@out \n\nCompleted with rc=\"$rc\".\n" ;
    Debug_Message("Archive finished with\n @out \n\nCompleted with rc=\"$rc\"") ;

    return $rc;

}

sub delete_target_dir {
    ($folder_to_delete) = @_;
    Debug_Message("Attempting to delete folder : $folder_to_delete") ;
    
    if($keep_log_open == 0)
        {
            Debug_Message("Attempted to close Log File. Changed to $startdir") ;
            CloseLogFile() ;
            chdir($startdir) ;
        }
        
        remove_tree($folder_to_delete) ;

    if ( -e "$folder_to_delete" ) {
        print "\n..Failed to delete $folder_to_delete in $startdir, please remove this yourself.\n";
            }
            else { 
                print "..Deleted temporary directory $folder_to_delete.\n";
            }
}
sub list_runtime_folder {
    
    if ( $tsm_available == 1 ) 
        {
        
            opendir( DIRHANDLE, $startdir ) || die ".. Unable to open folder $startdir";

            @files = grep !/^\.\.?$/, readdir DIRHANDLE;

            while (<@files>) 
                {
                    $rec = $_;

                    if ( $rec =~ /^(\d{4})(\d{2})(\d{2})-(\d{2})(\d{2})/ ) 
                        {
                            if ( $rec =~ /.zip$/ ) {

                                # Check and see if this zip file has been restored or just left hanging around;
                                if ( find_archive($rec) == 0 ) {
                                    Debug_Message("Unarchived ZIP file found : $startdir$rec") ;
                                    LogToBoth("..Unarchived ZIP file found in the source directory, attempting to archive\n") ;
                                    archive_zip_file("$startdir$rec");
                                }
                            }
                            else {                    
                                    if($rec ne $targetdir)
                                        {
                                            Debug_Message("Unarchived ZIP file found : $startdir$rec") ;
                                            LogToBoth(".. Unarchived folder found in the source directory ($rec), attempting to archive\n") ;
                                            $zip_file_to_archive = zip_report_files("$startdir$rec");
                                            archive_zip_file($zip_file_to_archive) ;
                                        }
                                    }
                        }
                }

            closedir(DIRHANDLE);
        }
        
        $keep_log_open = 0 ;
    }

sub find_archive 
{
    my $this_folder = @_;
    my $query_command = "$cmd -dataonly=yes \"SELECT COUNT(*) FROM archives WHERE TYPE='FILE' and LOWER(description) LIKE 'servermon: " . $arc_year . $arc_month  . $arc_day . "-" . $arc_hour . ":" . $arc_minute . "%'\"";

    if ( `$query_command` > 0 ) 
    	{
        	return 1;
    	}
    	else {
        		return 0;
    		 }
}

sub get_pid() {

    my @threadlist = `$cmd  SHOW THREADS`;

    foreach $thread (@threadlist) {

        print "$thread";

        if ( $thread =~ m/^Server PID:/ )
            {    # grab server pid from SHOW THREADS output
                @fields = split( /\s+/, $thread );
                return $fields[2];
            }

        if ( $thread =~ m/^  Server Version/ ) {    # check for server
            @fields = split( /\s+/, $thread );
            $version = $fields[3];
            $version =~ s/\D+//g;
            $release = $fields[5];
            $release =~ s/\D+//g;
            @lfields = split( /\./, $fields[7] );
            $level   = $lfields[0];
#            $slevel  = $lfields[1];

            if ( $sqlstats eq "Y" ) {
                if (
                       ( $version > 6 )
                    || ( ( $version == 6 ) && ( $release > 2 ) )
                    || (   ( $version == 6 )
                        && ( $release = 2 )
                        && ( $level >= 3 ) )
                  )
                {
                    $collectstmt = "collectstmt=true";
                    print "\n\"$collectstmt\" used for instrumentation statistics\n\n";
                }
                else {
                    print "\nInstrumentation trace does not support COLLECTSTMT option at your server level\n\n";
                    undef($collectstmt);
                }
            }
            else {
                undef($collectstmt);
            }

        }

        if (   ( $thread =~ m/^ANS8023E/ )
            || ( $thread =~ m/^ANS1217E/ )
            || ( $thread =~ m/^ANS0101E/ )
            || ( $thread =~ m/^ANS8019E/ ) )
        {    # Failure contacting the server/message repository error
            exit;
        }
    }
}

sub get_actlog_summary() 
	{
	    if ($actlog) 
	    	{
                    Debug_Message("Creating extract of Activity Log") ;
                    `$cmd -TAB query actlog begindate=$begindate begintime=$begintime > $actlogname`;

                    if ( -e "$actlogname" ) 
                        {
                            print "-- Wrote $actlogname\n";
                        }
                        else {
                            print "$actlogname not written!\n";
                        }
	    	}
	
        
                
#	    if ($summary) 
#	    	{
#	    		chdir("csv-files") ;
#	        	open( SUMMARY, ">$summaryname" ) || die "Can't open $summaryname for writing\n";
#	
#		        @out = `$cmd -tabd -dataonly=yes "select * from summary where start_time>=\'$begints\' or end_time>=\'$begints\'"`;
#		        print SUMMARY "@out\n\n";
#		        close(SUMMARY);
#		        chdir("..") ;
#		        
#		        print "-- Wrote $summaryname\n";
#	    	}


	}

sub get_instrumentation {
    
    if ($instrumentation =~ /y/i)
        {
            print ".. Attempting to stop any existing instrumentation from running, output will be saved\n";
            
            $filename = "instr-files/$_[0]";

            if ( traceActive() ) 
                {

                    open( INSTROUT, "> $filename" ) || exit;

                    print INSTROUT ".. Warning, server trace is active, there may be high \"Unknown\" time in instrumentation due to this and performance in general may be severely degraded because of trace.\n";
                    print INSTROUT ".. The performance impact of trace can be lessened on version 6+ by using \"bufsize=4096\" on the trace begin command when a crash is not expected.\n\n";

                    close(INSTROUT);

                }

            `$cmd instr end > $filename`;
            $rc = $?;

            if ( $rc == 0 ) { $instr_status = "Was running, successfully closed"; }
            elsif ( $rc == 6912 ) { $instr_status = "Was not active"; }
            else                  { $instr_status = $rc; }

            print "\n.. Instrumentation closed with status : $instr_status \n";

            $instractive = 0;

            print "-- Wrote $filename\n\n";
        } else {
                    LogToBoth("\n.. Instrumentation has been disabled via the commands.ini file") ;
                }

}

# verify if we are talking to a storage agent
sub is_sta {

    my @CheckSTA = `$cmd q status` ;

    foreach my $line (@CheckSTA) {

	        if ( $line =~ m/Server for this Storage Agent/i ) {
		        return 1;
	        } 
    }
}

# verify if SDPOOLS are enabled
sub hasSDpools {

    my $connected       = 0;
    my $retries         = 10;
    my $sleeptime       = 60;
    my $i               = 0;
    $hasSDpools_status  = 0;

    # Verifying if we are in an STA environment is the first server
    # connection attempt. We will retry and give up if we can't connect
    # after a number of retries

    while ( !($connected) && ( $i <= $retries ) ) {
        if ( !$i++ ) {
            print "\nVerify if container pools are supported.\n";
        }

        open( FILE, "$cmd  SHOW SDDIR |" ) || die "Can't open $cmd - $!";

        while (<FILE>) {
            if (m/^ANS8023E/) {
                print $_;
                print "Connect attempt " . $i . " failed";
                if ( $i <= $retries ) {
                    print ": will try again in "
                      . $sleeptime
                      . "s. ($retries retries)";
                }
                else {
                    print ".";
                }

                print "\n";

                sleep $sleeptime;

            }
            elsif ((m/^ANS1217E/)
                || (m/^ANS0101E/)
                || (m/^ANS1035S/)
                || (m/^ANS1051I/) )
            {    # Failure during session initialisation
                print $_;
                exit;

            }
            elsif (m/^ANR2000E/) {    # Unknown command - SHOW SDDIR.
                print $_;
                close(FILE);
                return (0);

            }
            elsif (m/^---------/) {
                $connected = 1;

            }
            elsif (m/^ANS0102W/)
            { # Unable to open the message repository <REPOSITORY> The American English repository will be used instead.
                print $_;

            }
            elsif ( (m/^ANS8000I/) || (m/^ANS8001I/) ) {
                print $_;

            }
            elsif (m/^ANS8002I/) {
                print $_;
                close(FILE);
                $hasSDpools_status = 1;
                return (1);

            }
            elsif (m/^ANS/) {    # some different message
                print $_;
                exit;
            }

        }
    }

    if ( $i < $retries ) {
        $hasSDpools_status = 1;
        return (1);
    }
    else {
        print "Giving up connecting to server, please check the server is up and very the parameters are correct:\nservername: \"$server\"\n";
        if (   ( $^O eq "MSWin32" )
            || ( $^O eq "Windows_NT" )
            || ( $^O eq "cygwin" ) )
        {
            print "tcpport   : \"$tcpport\"\n";
        }
        exit;
    }
}

# start trace only if no trace is currently running
sub startTrace() {

    # remove comma from traceflags and disableflags
    $traceflags   =~ s/,/ /g;
    $disableflags =~ s/,/ /g;


    #	Pre-allocate the trace file
    my $TraceFileName = getcwd . "/trace-files/$trcname" ;
    if (open my $fh, ">", $TraceFileName) {
        print "Allocated $TraceFileName\n" ;
        close $fh;
    } else {
        warn "Could not create : '$TraceFileName'. $!";
    }

    open( TRCCMDSOUT, ">trace-files/$trccmdname" )
      || die "Can't open TRCCMDSOUT \"trace-files/$trccmdname\" for writing\n";

    if ( traceActive() ) {
        print "Found active trace, tracing will not be activated.\n"; GeneralPrint() ;
        print TRCCMDSOUT "Found active trace, tracing not activated.";
        close(TRCCMDSOUT);
        return 0;
    }

    open( TRCCMDSOUT, ">trace-files/$trccmdname" )
      || die "Can't open TRCCMDSOUT \"trace-files/$trccmdname\" for writing\n";

    print "Reset traceflags.\n";
    print TRCCMDSOUT `$cmd "trace dis *"`;

    @out = `$cmd "trace ena $traceflags"`;
    
    print TRCCMDSOUT @out;

    foreach $line (@out) {
        if ( $line =~ /^ANR0927E/ ) {
            print $line;    # throw message
            close(TRCCMDSOUT);
            return 0;
        }
    }

    print "Enabled traceflag(s) $traceflags.\n";

    if ( ( $disableflags ne "" ) && ( uc($disableflags) ne "NONE" ) ) {
        @out = `$cmd "trace dis $disableflags"`;
        print TRCCMDSOUT @out;
        print "Disabled traceflag(s) $disableflags.\n";
    }

    if ( $maxsize > 0 ) {
        $limitsize = "maxsize=$maxsize";
    }
    else {
        $limitsize = "";
    }

    if (   ( $^O eq "MSWin32" )
        || ( $^O eq "Windows_NT" )
        || ( $^O eq "cygwin" ) )
    {
        @out = `$cmd trace begin $TraceFileName bufsize=4096 $limitsize`;
    }
    else {
        @out = `$cmd "trace begin $TraceFileName bufsize=4096 $limitsize"`;
    }

    print TRCCMDSOUT @out;

    foreach $line (@out) {

        if ( $line =~ /^ANR0921I/ ) {
            print $line;    # throw message
            close(TRCCMDSOUT);
            return 1;
        }

        if ( $line =~ /^ANR0928E/ ) {
            print $line;    # throw message
            print "Disabling current active traceflags again.\n";
            print TRCCMDSOUT
              `$cmd "trace dis *"`;    # disable traceflags specified above
            close(TRCCMDSOUT);
            return 0;
        }
    }

    close(TRCCMDSOUT);
    return 0;                          # should not get here...

}

sub stopTrace() {

    open( TRCCMDSOUT, ">>$trccmdname" ) || die "Can't open TRCCMDSOUT \"$trccmdname\" for writing\n";

    print TRCCMDSOUT `$cmd "trace end"`;
    print TRCCMDSOUT `$cmd "trace flush"`;
    print TRCCMDSOUT `$cmd "trace dis *"`;

    close(TRCCMDSOUT);

}

sub traceActive {

    @out = `$cmd q trace`;
    foreach $line (@out) {
        if ( $line =~ / active/ ) {
            return 1;
        }
    }

    return 0;

}

sub prepareDocdir {

    #
    #
    #

    $begindate = strftime( "%m/%d/%Y", localtime() );
    $begintime = strftime( "%H:%M:00", localtime() );
    $begints   = strftime( "%Y-%m-%d %H:%M:%S.000000", localtime() );
    $now       = strftime( "%Y%m%d-%H%M",  localtime() );

    $actlogname  = "$now-00000000-actlog.txt";
    $summaryname = "$now-00000000-summaryrec.csv";
    $trccmdname  = "$now-00000000-trace-commands.txt";
    $trcname     = "$now-00000000-trace.trc";
    
    my $pmr = "" ; # Temp entry

    $cyclesCompleted = 0;

    $targetdir = $now . "-" . $serverName . "-swg21432937";

    #
    #
    #

    if ( -d $targetdir ) {
        print "Target directory $targetdir exists, exiting";
        exit;
    }
    elsif ( -f $targetdir ) {
        print "A file with the name of the target directory $targetdir exits, exiting"; generalPrint() ;
        exit;
    }
    else {
        mkdir($targetdir) || die "Could not create $targetdir in $cwd";
        chdir($targetdir);
        $rundir = getcwd;
                
        MessagesLogHandler(0) ;	#	Create Log handler into current directory
        LogToBoth("Servermon Version: $version\n\n") ;
        LogToBoth("Created target directory $rundir, starting to collect docs.\n\n") ;
        
        #
        #	Create various folders
        #

        mkdir("csv-files")  ;
        mkdir("show-files") ;
        mkdir("vmstat-files") ;
        mkdir("iostat-files") ;
        mkdir("ini-files") ;
        mkdir("db2-files") ;
        mkdir("db2pd-files") ;
        mkdir("corrlogtxn-files") ;
        mkdir("instr-files") ;
        mkdir("lparstat-files") ;
        mkdir("trace-files") ;
        mkdir("ffdc-files") ;        
        
        # create doc collection folders
        mkdir("autopdzip");
        chdir("autopdzip");
        mkdir("autopd");
        chdir("autopd");
        open( XMLOUT, "> autopd-collection-environment-v2.xml" ) || die "Can't open autopd-collection-environment-v2.xml for writing\n";
        print XMLOUT "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n" . "<collectionEnvironmentInfo pluginTaxonomyId=\"SSGSG7\" toolName=\"" . $filename . "\" toolVersion=\""  . $version . "\"\n";
          
        if ( $instance ne "" ) {
            print XMLOUT "   DB2Instance=\"$instance\"\n";
        }

        my $login = "";
        print XMLOUT "   osName=\"$^O\"\n";

        print XMLOUT "   hostname=\"" . chomp($hostname) . "\"\n";

        if ( ( $^O eq "MSWin32" ) || ( $^O eq "Windows_NT" ) ) {

            my ( $osVername, $osMajor, $osMinor, $osId ) =
              Win32::GetOSVersion();
            print XMLOUT "   osVersion=\"$osVername\" osMajor=\"oaMajor\" osMinor=\"$osMinor\" osId=\"$osId\"\n";

            $login = Win32::LoginName();
        }
        
        if ( $login ne "" ) {
            print XMLOUT "   userName=\"$login\"\n";
        }

        print XMLOUT "   showthreads=\"$showthreads\"\n";
        print XMLOUT "   showdedupdelete=\"$showdedupdelete\"\n";
        print XMLOUT "   showmemory=\"$showmemory\"\n";
        print XMLOUT "   showreplication=\"$showreplication\"\n";
        print XMLOUT "   sqlstats=\"$sqlstats\"\n";
        print XMLOUT "   dynamicsql=\"$dynamicsql\"\n";
        print XMLOUT "   monitorlogspan=\"$monitorlogspan\"\n";
        print XMLOUT "   traceflags=\"$traceflags\"\n";
        print XMLOUT "   disableflags=\"$disableflags\"\n";
        print XMLOUT "   maxsize=\"$maxsize\"\n";
        print XMLOUT "   stopevent=\"$stopevent\"\n";
        print XMLOUT "   ffdccopy=\"$ffdccopy\"\n";
        print XMLOUT "   dbalias=\"$dbalias\"\n";
        print XMLOUT "   sleeptime=\"$sleeptime\"\n";
        print XMLOUT "   iterations=\"$iterations\"\n";
        print XMLOUT "   repeat=\"$repeat\"\n";
        print XMLOUT "   continuous=\"$continuous\"\n";
        print XMLOUT "   optfile=\"$optfile\"\n";
        print XMLOUT "   deletefiles=\"$deletefiles\"\n";
        print XMLOUT "   archmc=\"$archmc\"\n";
        print XMLOUT "   IT21957=\"$IT21957\"\n";

        print XMLOUT "xmlns=\"http://www.ibm.com/autopd/collectionEnvironment\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"http://www.ibm.com/autopd/collectionEnvironment ../autoPD-collection-env.xsd\" />";
        close(XMLOUT);
        chdir($rundir);

        return 1;
    }

}
#
# start correlateTxnLog functions
#
sub correlateTxnLog() {

    my $db2pdFile = $_[0];

    # print("\n\n+++++++++++++++++++++++ $db2pdFile +++++++++++++++++++++++\n");

    $corrLogTxnFile = "corrlogtxn-files/" . $prefix . "-corrlogtxn.txt";
    open( CORRLOGTXNOUT, ">$corrLogTxnFile" ) || die "Can't open CORRLOGTXNOUT for writing\n";

    my @transFileAr = &getArray( $db2pdFile, "Transactions" );
    my @logFileAr   = &getArray( $db2pdFile, "Logs" );
    my $transSize   = @transFileAr;
    my $logSize     = @logFileAr;

    $filesChecked++;

    # Print out the file record containing the timestamp and figure out if
    # we're running DB2 9.7 or before or DB2 10.5
    my $is710 = &getId( "Transactions", @transFileAr );
    &getId( "Logs", @logFileAr );

    # Names of the log files and the LSN or LSO values
    my @logName  = ();
    my @startVal = ();
    my $logIdx   = 0;    # index for these arrays

    # Size of the last log file
    my $lastLogSize = 0;

    my $idx = 0;

    # Figure out the startLsn from the first line that has a log file in it
    # Store the log file names in the array logName.
    for ( $idx = 0 ; $idx < $logSize ; $idx++ ) {
        my $thisLine = $logFileAr[$idx];
        chop $thisLine;

        # If there's whitespace at the beginning of the line, delete it.
        if ( $thisLine =~ /^(\s+)0x.+LOG$/ ) {
            my $saveWhiteSpace = $1;
            my $thisLen        = length($saveWhiteSpace);
            $thisLine = substr( $thisLine, $thisLen );
        }

        # Ignore all lines except those that start with 0x and end with LOG
        if ( $thisLine =~ /^0x.+LOG$/ ) {
            my @splitLine   = split( /\s+/, $thisLine );
            my $startValCol = 1;
            my $logSizeCol  = 3;
            my $fileNameCol = 5;

            if ($is710) {
                $startValCol++;
                $logSizeCol++;
                $fileNameCol++;
            }

      #         print "$splitLine[ $startValCol ] $splitLine[ $fileNameCol ]\n";
            $logName[$logIdx] = $splitLine[$fileNameCol];

            no warnings;
            $startVal[$logIdx] = (
                  $is710
                ? $splitLine[$startValCol]
                : hex( $splitLine[$startValCol] )
            );
            warnings;

            $lastLogSize = $splitLine[$logSizeCol];
            $logIdx++;
        }
    }

    # Add a pseudo-log file at the end of the array so we can account for
    # the addresses in the last log.
    {
        $logName[$logIdx] = "PLACEHOLDER";

        $startVal[$logIdx] = $startVal[ $logIdx - 1 ] + ( $lastLogSize * 4096 );
#        $myString = &myHex( $startVal[$logIdx] );

        # print "logIdx = $logIdx, startval = $myString\n";
        $logIdx++;
    }

    # Print out the first and last log file names.
    print CORRLOGTXNOUT "$db2pdFile: FirstLog=$logName[ 0 ], lastLog=$logName[ $logIdx-2 ]\n";

    # Now go through the transaction file finding the WRITE transactions and
    # acting on them.
    my $cumSpaceReserved = 0;
    my $cumLogSpace      = 0;
    for ( $idx = 0 ; $idx < $transSize ; $idx++ ) {
        my $thisLine = $transFileAr[$idx];

        # If there's whitespace at the beginning of the line, delete it.
        if ( $thisLine =~ /^(\s+)0x[^W]+WRITE/ ) {
            my $saveWhiteSpace = $1;
            my $thisLen        = length($saveWhiteSpace);
            $thisLine = substr( $thisLine, $thisLen );
        }

# Ignore all lines except those that start with 0x and have WRITE somewhere in there
        if ( $thisLine =~ /^0x[^W]+WRITE/ ) {
            my @splitLine = split( /\s+/, $thisLine );

            my $appIdCol         = 1;
            my $firstValCol      = 8;
            my $lastValCol       = 9;
            my $spaceReservedCol = 10;
            my $logSpaceCol      = 11;

            if ($is710) {
                $firstValCol      += 2;
                $lastValCol       += 2;
                $spaceReservedCol += 2;
                $logSpaceCol      += 2;
            }

            my $appId = $splitLine[$appIdCol];

            no warnings;
            my $firstVal = (
                  $is710
                ? $splitLine[$firstValCol]
                : hex( $splitLine[$firstValCol] )
            );

            my $lastVal = (
                  $is710
                ? $splitLine[$lastValCol]
                : hex( $splitLine[$lastValCol] )
            );
            warnings;

            my $spaceReserved = $splitLine[$spaceReservedCol];
            my $logSpace      = $splitLine[$logSpaceCol];

            $firstLog = &getLog( $firstVal, @startVal );
            $lastLog  = &getLog( $lastVal,  @startVal );

            # print the information for this WRITE transaction out
            $loLog = &ExtractLog( $logName[$firstLog] );
            $hiLog = &ExtractLog( $logName[$lastLog] );
            my $logsSpanned = $hiLog - $loLog;

            if ( $logsSpanned < 0 )
                {   # reset if $logsSpanned gets negative (logs not initialized...)
                    $logsSpanned = 0;
                }

            print CORRLOGTXNOUT  "$db2pdFile: applHandle $appId firstLog $logName[ $firstLog ] " . "lastLog $logName[ $lastLog ] spaceReserved $spaceReserved " . "logSpace $logSpace logsSpanned $logsSpanned\n";
            $cumSpaceReserved += $spaceReserved;
            $cumLogSpace      += $logSpace;

            if ( $logsSpanned > $spanmax ) 
                {
                    $spanmax      = $logsSpanned;
                    $spanmaxFile  = $db2pdFile;
                    $spanmaxAppId = $appId;
                }

        }
    }

    print CORRLOGTXNOUT "$db2pdFile: cumSpaceReserved $cumSpaceReserved cumLogSpace $cumLogSpace\n";

    if ($spanmaxAppId) 
        {
            print CORRLOGTXNOUT "\n$db2pdFile: $spanmaxFile reports the applHandle $spanmaxAppId spanning $spanmax log files. Evaluated $filesChecked files.\n";
        }

    print "-- Wrote $corrLogTxnFile\n";
    close(CORRLOGTXNOUT);
}    # sub correlateTxnLog()

# Read the whole file into an array and return it.
sub getFile() {
    my $fileName  = $_[0];
    my @theValues = ();
    my $inHandle  = new IO::File;
    open( $inHandle, $fileName ) || die("cannot open $fileName\n");
    @theValues = <$inHandle>;
    close $inHandle;

    @theValues;

}    #sub getFile

sub ExtractLog {
    $_ = $_[0];
    /[S](\d+).+/;
    my $logNum = $1;
    $logNum;
}    # sub getFile()

sub getId() {
    my $prtLabel = $_[0];
    my @result   = grep { /Database TSMDB1 -- Active/ } @_;
    my $thisSize = $#result;

    my $is710 = 0;
    if ( $thisSize == 0 ) {

        # Figure out if we're running db2 10.5
        if ( $result[0] =~ /Database Member / ) {
            $is710 = 1;
        }
    }
    else {
        # print "First line of $prtLabel:\n";
        # print "\t$result[1]";
    }

    # return whether or not DB2 10.5 running
    $is710;
}    # sub getId()

sub getLog {
    my @startVal        = @_;
    my $valueOfInterest = $startVal[0];

    my $logIdx = $#startVal;
    my $idz    = 1;

    # Now figure out what logfile the value is in.
    while ( $valueOfInterest > $startVal[$idz] && $idz < $logIdx ) {
        $myString1 = &myHex($valueOfInterest);
#        $myString2 = &myHex( $startVal[$idz] );

        # print "$idz, $myString1, $myString2\n";
        $idz++;
    }

    # If we didn't find it yet, check to see if it's in the last page
    if ( $idz == $logIdx ) {
        if ( $valueOfInterest >= $startVal[$idz] ) {
            my $myString1 = &myHex($valueOfInterest);
            die("Couldn't find logfile for $valueOfInterest $myString1.\n");
        }
    }

    # Remember we went 1 too far in the loop, and $valueOfInterest
    # took the first loc in the array.
    $idz - 2;
}    # sub getLog()

sub myHex() {
    my $value = $_[0];
    my $outString = sprintf( "0x%.16x", $value );
    $outString;
}    # sub myHex()

sub getArray() {

    my $fileName     = $_[0];
    my $searchstring = $_[1];
    my @theValues    = ();
    my $inHandle     = new IO::File;
    my $dbactive     = 0;
    my $command      = $found1 = $found2 = 0;
    open( $inHandle, $fileName ) || die("cannot open $fileName in getTxn\n");

    # while (<$inHandle>) {

    foreach $line (<$inHandle>) {

        if ( $line =~ /db2pd -db $dbalias -transactions -logs$/ ) {
            $command = 1;
        }

        if ( !($dbactive) && ( $line =~ /Database TSMDB1 -- Active/ ) ) {
            push( @theValues, $line );
            $dbactive = 1;
        }

        if ($command) {

            if ( $line =~ /$searchstring/ ) {
                $found1 = 1;
            }
            elsif ( ($found1) && ( $line =~ /Address/ ) ) {
                $found2 = 1;
            }

            if ( $found1 && $found2 ) {

                push( @theValues, $line );
                chomp $line;

                if ( $line =~ /^\s*$/ ) {
                    close $inHandle;
                    return @theValues;
                }

            }
        }
    }

    close $inHandle;
    return @theValues;

}    #sub getArray

sub getTimeStamp
{
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
    my $nice_timestamp = sprintf ( "%02d:%02d:%02d", $hour,$min,$sec);
    
    return $nice_timestamp;
}

sub zip_or_copy_file() {
# zip_or_copy_file($logfile.$idx, $insthome, $ffdcDir);
                         
   my $file       = $_[0];
   my $fromdir    = $_[1];
   my $todir      = $_[2];

   $fromdir =~ s!/*$!/!; # Add a trailing slash
   $todir   =~ s!/*$!/!; # Add a trailing slash
    
   my $source = $fromdir . $file;
   my $target = $todir . $file;
   
   $db2ZipProgram = getDB2JavaBin();
           
   if ( $db2ZipProgram ne "No_DB2" ) 
   {
       # zip file

       my $rundir = getcwd() . "/";       
       chdir("$fromdir");
           
       my $zipCommand   = "$db2ZipProgram -cvMf " . $rundir . $target . ".zip ./$file";
       print "$zipCommand\n";
       my @callZip      = `$zipCommand`;    
       print @callZip;

       chdir($rundir);

   }
       else 
   {
       # copy file
       if ( copy( $source, $target ) ) {
          print "\n-- Copied $source to $target directory.\n";
       }
   }
}

sub SetSpectrumProtectPaths
{
	if ( ( $^O eq "MSWin32" ) || ( $^O eq "Windows_NT" ) || ( $^O eq "cygwin" ) ) 
            {
                $pathdelimiter = ";" ;
                
                my $tsmpathreg = "HKEY_LOCAL_MACHINE/SOFTWARE/IBM/ADSM/CurrentVersion/BackupClient" ;
                my $regKey  = new Win32::TieRegistry $tsmpathreg, { Access => KEY_READ(), Delimiter => "/" } ;
                $tsmpath   = $regKey->GetValue("Path") . "\\";

                $cmd    = "dsmadmc -tcps=$server -tcpp=$tcpport -id=$administrator -pas=\"$password\" -scrollprompt=no -dateformat=1 -numberformat=1 -errorlogname=\"$startdir/dsmerror.log\"" ;
                $cmdCsv = "dsmadmc -tcps=$server -tcpp=$tcpport -id=$administrator -pas=\"$password\" -scrollprompt=no -dateformat=1 -numberformat=1 -$delimeter -dataonly=yes -errorlogname=\"$startdir/dsmerror.log\"" ;

                Debug_Message("Windows CommandCSV is set to : $cmdCsv") ;

                if ( $ENV{'DSM_CONFIG'} eq "" ) 
                    {
                        $dsm_opt = "$tsmpath\\dsm.opt";
                    }
                    else {
                            $dsm_opt = $ENV{'DSM_CONFIG'};
                         }

                if ( $ENV{'DSM_DIR'} ne "" ) 
                    {
                        $tsmpath = $ENV{'DSM_DIR'} ;
                    }
                    
                my $tsmdb2pathreg = "HKEY_LOCAL_MACHINE/SOFTWARE/IBM/DB2/GLOBAL_PROFILE" ;
                my $regKey  = new Win32::TieRegistry $tsmdb2pathreg, { Access => KEY_READ(), Delimiter => "/" } ;
                $db2path   = $regKey->GetValue("DB2PATH") . "\\";

                $db2cmdpath   = $db2pdcmdpath = $db2path ;

            }
	else {    #UNIX

                $pathdelimiter = ":";

                $cmd    = "dsmadmc -servername=$server -id=$administrator -pas=\"$password\" -dateformat=1 -numberformat=1 -scrollprompt=no  -errorlogname=\"$startdir/dsmerror.log\"";
                $cmdCsv = "dsmadmc -servername=$server -id=$administrator -pas=\"$password\" -dateformat=1 -numberformat=1 -scrollprompt=no  -$delimeter -dataonly=yes -errorlogname=\"$startdir/dsmerror.log\"";
                
                Debug_Message("Linux CommandCSV is set to : $cmdCsv") ;

                if ( $^O eq "aix" )
                     {
                        $fsclient = "/usr";
                        $fsserver = "/opt";

                        if ( $ENV{'DSM_CONFIG'} eq "" ) 
                            {
                                $dsm_opt = $fsclient . "/tivoli/tsm/client/ba/bin64/dsm.opt";
                            }
                            else {
                                    $dsm_opt = $ENV{'DSM_CONFIG'};
                                    }

                        if ( $ENV{'DSM_DIR'} eq "" ) 
                            {
                                $tsmpath = $fsclient . "/tivoli/tsm/client/ba/bin64/";
                            }
                                else {
                                            $tsmpath = $ENV{'DSM_DIR'};
                                     }

                        $procstackbin = "/usr/bin/procstack";
                        $procrun      = "/usr/bin/procrun";

                    #     $statsiterations = max(1, int( $iterations * $sleeptime / $statsinterval));
                          $statsiterations = max( 1, int( ( $iterations - 1 ) * $sleeptime / $statsinterval ) )  ;    # DJD

                          $statscmd         = "iostat -DlT $statsinterval $statsiterations";
                          $vmstatcmd        = "vmstat -Itw $statsinterval $statsiterations";
                          $lparstatcmd      = "lparstat $statsinterval $statsiterations";
                          $lparstatcmdfinal = "lparstat -i";
                     }
                    else {
                            $fsclient = "/opt";
                            $fsserver = "/opt";

                            if ( $ENV{'DSM_CONFIG'} eq "" ) 
                                {
                                    $dsm_opt = $fsclient . "/tivoli/tsm/client/ba/bin/dsm.opt";
                                }
                                else {
                                        $dsm_opt = $ENV{'DSM_CONFIG'};
                                     }

                            if ( $ENV{'DSM_DIR'} eq "" ) 
                                {
                                    $tsmpath = $fsclient . "/tivoli/tsm/client/ba/bin/";
                                }
                                    else {
                                            $tsmpath = $ENV{'DSM_DIR'};
                                         }

                            if ( ( $^O eq "linux" ) || ( $^O eq "solaris" ) ) 
                                {
                                    if ( -x "/usr/bin/pstack" ) 
                                            {
                                                $procstackbin = "/usr/bin/pstack";
                                            }

                                    if ( -x "/usr/bin/gstack" ) 
                                        {
                                            $procstackbin = "/usr/bin/gstack";
                                        }
                                }

                              #	 $statsiterations = max(1, int( $iterations * $sleeptime / $statsinterval));
                                    $statsiterations =
                                      max( 1, int( ( $iterations - 1 ) * $sleeptime / $statsinterval ) )
                                      ;    # DJD

                            if ( $^O eq "linux" ) 
                                {
                                    if ( -x "/usr/bin/iostat" ) 
                                        {
                                            $statscmd = "/usr/bin/iostat -xtk $statsinterval $statsiterations";
                                        }

                                    if ( -x "/usr/bin/vmstat" ) 
                                        {
                                            $vmstatcmd = "/usr/bin/vmstat -tw $statsinterval $statsiterations";
                                        }
                                    if (-x "/usr/bin/nfsiostat" ) 
                                        {
                                            $nfsstatscmd = "/usr/bin/nfsiostat $statsinterval $statsiterations";
                                        }
                                }

                            if ( $^O eq "solaris" ) 
                                {
                                    $statscmd  = "iostat -xnT d $statsinterval $statsiterations";
                                    $vmstatcmd = "vmstat $statsinterval $statsiterations";
                                }

                            if ( $^O eq "hpux" ) 
                                {
                                    $statscmd  = "iostat -t $statsinterval $statsiterations";
                                    $vmstatcmd = "vmstat $statsinterval $statsiterations";
                                }
				
                        }

	    $db2cmdpath   = $fsserver . "/tivoli/tsm/db2/bin";
	    $db2pdcmdpath = $fsserver . "/tivoli/tsm/db2/adm";
}

my $path = $ENV{'PATH'};
$ENV{'PATH'} = $path . $pathdelimiter . $tsmpath . $pathdelimiter . $db2cmdpath . $pathdelimiter  . $db2pdcmdpath ;
  
$ENV{'DSM_DIR'}    = $tsmpath;
$ENV{'DSM_CONFIG'} = $dsm_opt;
	
}

sub WriteServerMonHeader {
	
	for my $header (1..2)
		{
                    MessagesLogHandler($header) ;

                    print "**********************************************************************\n";
                    print "* (C) Copyright IBM Corporation 2014, 2016.  All rights reserved.    *\n";
                    print "*                                                                    *\n";
                    print "* IBM Spectrum Protect                                               *\n";
                    print "*                                                                    *\n";
                    print "* This script collects useful information to verify server           *\n";
                    print "* operations as documented with technote swg21432937.                *\n";
                    print "* See http://www-01.ibm.com/support/docview.wss?uid=swg21432937      *\n";
                    print "* for the latest version of the script                               *\n";
                    print "* You need to run the script as instance user with the DB2 env       *\n";
                    print "* initialized. The script will prompt you for the necessary          *\n";
                    print "* parameters required to run successfully.                           *\n";
                    print "*                                                                    *\n";
                    print "* Usage: "  . $filename 	  . " <noprompt>                        *\n";
                    print "*                                                                    *\n";
                    print "* Dependencies: UNIX or Windows                                      *\n";
                    print "*               Perl interpreter installed                           *\n";
                    print "*               IBM Spectrum Protect administrative client installed *\n";
                    print "*               and configured                                       *\n";
                    print "*               IBM Spectrum Protect server installed on the box     *\n";
                    print "*                                                                    *\n";
                    print "* "   . $filename . " " . $version . "                               *\n";
                    print "*                                                                    *\n";
                    print "**********************************************************************\n";
		
		}		
}

sub MessagesLogHandler
{
	my $LogSetting = $_[0] ;
	
	if($LogSetting == 0)
		{
                    open( $SERVERMONLOG, '>>', "$now-99999999-servermon-log.txt" );	# Dowe need to add targetdir?			
		}
		
	if($LogSetting == 1)
		{
                    select(STDOUT) ;
		}		
		
	if($LogSetting == 2)
		{
                    select($SERVERMONLOG) ;									
		}
}

sub LogToBoth
    {
        my $LogMessage     = $_[0];

        select(STDOUT) ;

        if($writetolog eq "Y")
                {
                    for my $header (1..2)
                        {
                                MessagesLogHandler($header) ;
                                print "$LogMessage\n" ;
                        }
                } else {			
                            MessagesLogHandler(1) ;
                        }	
    }

sub CloseLogFile
    {
        Debug_Message("Closing the log file") ;
        MessagesLogHandler(1) ;
        close( $SERVERMONLOG ) ;
    }

sub GeneralPrint()
    {
     if($writetolog eq "Y")
            {
                MessagesLogHandler(1) ;
                print ".." ;
                MessagesLogHandler(2) ;	
            }
    }


sub Debug_Message()
    {
        if ($Debug == 0 )
            {
                return
            }

        local ($messageIn) = @_ ;
        (my $newMessageIn = $messageIn) =~ s/\Q$password\E/********/g ;

        my @now = localtime();
        my $timeStamp = sprintf("%04d-%02d-%02d%- 02d:%02d:%02d", $now[5]+1900, $now[4]+1, $now[3],$now[2], $now[1], $now[0]);

        my $messageOut = "--$timeStamp : $newMessageIn\n" ;                      

        print($messageOut) ;    
    }
    
sub build_credentials
    {
		read_ini_files("$startdir", "servermon.ini");
        
		requestCredentials() ;
        getCredentials();
        
        goto START
    }
    
sub show_details
    {
        $Debug = 1;
        print "Detailed Log Information Has Been Turned On\n" ;
    }
    
sub prompt
    {
        $prompt = 1 ;
        print "Console input is active\n" ;
    }
 
sub getCredentials
    {    
        open( FH, '<', 'servermon.pwd') or die "Unable to open pwd file\n" ;
        my $readLine = <FH> ;
        close FH ;
        
        my $counter = 1 ;
        my @DecryptCredentials = split /\xc8/, $readLine ;
      
        $administrator  = decrypt($DecryptCredentials[0]) ;
        $password       = decrypt($DecryptCredentials[1]) ;
    }

sub requestCredentials
    {
		MessagesLogHandler(1) ;
        $administrator = getResp( "Enter the IBM Spectrum Protect administrator login ID ", $administrator );        
        $password     = readPassword("Enter Spectrum Protect Password ", $password) ;
                
        my $encrypted_credentials = encrypt($administrator) . chr(200) . encrypt($password);
        
        open(FH, '>', 'servermon.pwd') ;
        print FH $encrypted_credentials ;
        close FH ;
		
		SetSpectrumProtectPaths() ;
		
		if ( testSpectrumProtectAdminConnection(1) == 2)
			{
				#print("There has been an issue accessing the required Spectrum Protect server. The most likely reasons are : \n\t1) The username and/or password is incorrect\n\t2) The dsm.opt file might not exist, be accessible or have an incorrect parameter\n");
				print("\n\nThe supplied credentials will now be deleted and ServerMon will exit\n\n") ;
				
				unlink("servermon.pwd") ;
				exit 1;
			}

		
		MessagesLogHandler(2) ;
    }

sub encrypt
    {  
        ($StringIn) = @_ ;
        $ReturnString = "" ;
        $counter  = 1 ;
        $char_offset = 23 ;

        foreach my $char( split //, $StringIn)
            {
                my $NewValue = ( ord($char) + $char_offset ) + $counter ;
                $ReturnString = $ReturnString . chr($NewValue) ;
                $counter++ ;
            }
        
        return $ReturnString ;   
   }

sub decrypt
{   
    ($StringIn) = @_ ;
    $counter = 1 ;
    $ReturnString = "" ;
    $char_offset = 23 ;
    
    foreach my $char(split //, $StringIn)
        {               
            $ReturnString = $ReturnString . chr((ord($char)- $char_offset)-$counter) ;
            $counter++ ;
        }
        
    return $ReturnString ; 
}

sub testSpectrumProtectAdminConnection
    {
		
		($freshCredentials)=@_;
        my $maxReturnCode=1;
		my $fileName = "check_spectrum_protect_access.txt" ;
        my $CheckSP = "$cmd -scrollprompt=no quit > $fileName" ;       
        my @TestSP = `$CheckSP 2>&1`;
					
		MessagesLogHandler(1) ;
		
        open(CHECKSP1, $fileName) ;
        
		while(my $lineIn = <CHECKSP1>)
                {							
                    @thisLine = split(' ', $lineIn) ;
					push(@fileContents, $lineIn) ;

                    if ( $thisLine[0] eq 'ANS8002I' ) 
                        {
                                $returnCode = $thisLine[5] ;
                                if ($returnCode == 0 ) 
                                    {
										$maxReturnCode =  0;
										last ;
                                    }
						}

					if ($lineIn =~ /^Session established with server/)
						{
							$serverName = $thisLine[4] ;
							chop($serverName) ;
						}

				}
		
#								if ($returnCode == 137)
#									{            
#										print "\n\nServerMon is unable to access the Spectrum Protect server using the supplied username/password. " .
#											  "Further information can be found in the log file \"$fileName\, which is located in the ServerMon folder. Please rerun Servermon in prompt mode and supply new credentials or rerun " . 
#											  "Servermon with 'perl servermon.pl buildcredentials' and supply new credentials.\n\n" ;        
										
#										last ;
#									}
									
#								if ( $returnCode == -53 )
#									{            
#										print "\n\nServerMon is unable to access the Spectrum Protect server using the dsm.opt/sys stanza specified in the 'servermon.ini' file. Further information can be found in the log file \"$fileName\"" . 
#									  ", which is located in the ServerMon folder. Please verify the setting and rerun ServerMon\n\nServerMon will now exit\n\n" ;
#										last;
#									}
									
#								if($returnCode != 0)
#									{
#										print "\n\nServerMon is unable to access the Spectrum Protect server. Thie most likely reasons are \n\t1) Incorrect definitions in the servermon.ini [TSM] stanza.\n\t2) The supplied username or password is incorrect.\n " ;
#										print "\n\nServerMon will now exit\n" ;
#										last ;
#									}
#						}

#				if ( $thisLine[0] eq 'ANS1035S' )
#					{
#						print("\n\nServerMon has been unable to open or find the Spectrum Protect options please. Please check the setting in servermon.ini\n") ;
#						$maxReturnCode = 1 ;
#					}
						


				#}
				

		if ($maxReturnCode == 1)
			{
				print("\n\n* ************** ServerMon has been unable to connect to the supplied server. The log shows...... **********\n\n") ;
				
				foreach (@fileContents) 
					{
						print "* $_";
					}
					
				print("\n\n* ************** End of log *****************\n\n") ;
				
				if ( $maxReturnCode != 1)
					{
						exit 4 ;
					}
			}
						
        open(my $FH, '<', $filename);   
				
								
		if ( $freshCredentials == 1 & $maxReturnCode == 1)
			{
				return 2;
			}

        return $maxReturnCode ;

    }
    
sub checkInbuiltServerMonStatus
    {

        if ($storageagent == 1 ) # If a storage agent is enabled a "Q OPT ALWAYSSERVERMON" will show as active
            {
                return ;  
		    }

        my $ServerMonAlwaysOnStatus = "$cmd -dataonly=yes -scrollprompt=no q opt alwaysonservermon" ;
        my @CheckAlwaysOn = `$ServerMonAlwaysOnStatus` ;
        my $LastLine =  @CheckAlwaysOn[$#CheckAlwaysOn] ;

        if  ( $LastLine =~ /Yes/ ) 
            {
                    print "\n\nThe PERL Servermon Utility cannot run alongside ServerMon internal\n" ;
                    print "The utility will now exit\n" ;
                    exit 1;
            } 
    }
    
sub Check_DB2
	{
            my @db2ilist = `db2ilist`;

            $rc = $?;
            if ($rc) 
                {
                    print "ERROR: db2ilist returns $rc, are you running as instance user with the environment properly sourced?\n\n";
                    exit;
                }

            my $instances = "";
            my $comma     = "";

            foreach $line (@db2ilist) 
                {
                    chomp $line;

                    if (   ( $^O eq "MSWin32" ) || ( $^O eq "Windows_NT" )  || ( $^O eq "cygwin" ) )
                        {
                            # in a cluster the db2ilist looks like "INSTANCE CLUSTERRESOURCE"
                            my @clusterarr = split /\s+/, $line;
                            $line = $clusterarr[0];
                        }

                    if ( $line =~ /^$instance/ ) 
                        {
                            print("Instance found in db2ilist, using $instance\n");
                            $ENV{"DB2INSTANCE"} = $instance;
                            $instancefound = 1;
                        }

                    $instances .= $comma . $line;
                    $comma = ", ";
                }

            if ( !$instancefound ) {
                LogToBoth("\nInstance \"" . $instance . "\" not found in db2ilist, available: $instances. Please update \"servermon.ini\" or renter from the terminal if running interactivily\n") ;

                # in noprompt mode we should have a valid instance here
                if (!$prompt) {
                    exit;
                }
            }
	}

sub RootMessageHeader
    {
        LogToBoth("**********************************************************************" );
        LogToBoth("*                                                                    *") ;
        LogToBoth("*  ServerMon will now run the 'Administration' mode                  *") ;
        LogToBoth("*     1) All commands in the [root-only] will be executed           *") ;
        LogToBoth("*     2) ServerMon will exit once all commands have finished         *") ;
        LogToBoth("*                                                                    *") ;
        LogToBoth("*  Please switch to the DB2 user and rerun ServerMon as normal       *") ;
        LogToBoth("*                                                                    *") ;
        LogToBoth("**********************************************************************\n") ;
    }

sub findRootCommandFiles
{
    my $deleteFiles = @_ ;

    opendir(DIR, $startdir);
    @files = grep(/-00000000-root-commands.txt$/,readdir(DIR));  
    closedir(DIR);
    my $fileName = "" ;

    if ( $deleteFiles == 1)
        {
            my $latestRootOutputFile = pop @files ;  
			LogToBoth("\nPreserving $latestRootOutputFile") ;
		}

    foreach $file (@files) {
         $fileName = $file;
		 
         if ($deleteFiles == 1)
            {
                unlink "$startdir$fileName" ;     
			}
        }

       return $fileName ;
}